/*********************************************************
Awareness.
This is the parent class of all awareness measurement tools.
Any one can implement his(her) own awareness by inheriting
this class. 
The implemented awareness tools could be a DLL, so that our
middleware can load the dll dynamically, just like components
There is only one instance of each awareness tool in our 
middleware. However, this measurement tool can notify multiple
conditioner.
*********************************************************/
#pragma once

#define MAX_VALUE_SIZE	1024

#define DATA_SOURCE_DS_SERVER	"DirectoryServer"
#define DATA_SOURCE_SHARED_TOOL	"SharedTool"
#define DATA_SOURCE_INDIV_TOOL	"IndividualTool"
#define DATA_SOURCE_CONFIG_FILE	"File"

#define CONFIG_FILE_PARAM_FILE_DIR	"fileDir"
#define CONFIG_FILE_PARAM_FILE_AREA	"fileArea"
#define CONFIG_FILE_LENGTH_NAME		"Len"
#define CONFIG_FILE_TIME_PREFIX		"t"
#define CONFIG_FILE_VALUE_PREFIX	"v"

#include <afxmt.h>


typedef CArray<HANDLE, HANDLE&> HANDLEArray;

class CAware
{
public:
	CAware(void);
public:
	~CAware(void);

public:
	virtual void SetParam(char *, char *) {return;}
	virtual void Start(){return;}
	virtual void Stop() {return;}

public:
	void SetSource(char * source);
	bool SetValue(char *, int);
	bool GetValue(char *, int *);

	void AddNotifier(HANDLE hNotifer);
	void NotifyConditioner();
	bool IsStarted();

protected:
	static CCriticalSection m_dataLock;
	char m_chValue[MAX_VALUE_SIZE];
	int  m_nLength;
	bool m_bDataReady;
	bool m_bStarted;

	HANDLEArray m_hNotifiers;
	char s_source[255];
};

typedef CAware * CAwarePtr;