/*********************************************************
Component.
This is the parent class of all reconfigurable components.
Any one can implement his(her) own components by inheriting
this class. 
The implemented components could be a DLL, so that our
middleware can load the dll dynamically, just like awareness
measurement tools
There is only one instance of each component in our middleware.
All the actuators or metaobject just get a pointer pointing to
the component.
*********************************************************/
#pragma once
#include <Afxtempl.h>

#define MAX_PARAMETER_NUM 10
#define DllExport __declspec( dllexport )
#define DllImport __declspec( dllimport )

class CComponent
{
public:
	CComponent(void);
public:
	virtual ~CComponent(void);
public:
	bool input(void * buf, size_t len);
	bool output(void ** buf, size_t * len);
	virtual bool setparam(char * key, char * value) {return false;}
	virtual bool start() {return false;}
	virtual bool stop() {return false;}
public:
	void SetName(char * name);
	char * GetName();
	void SetDllInst(HINSTANCE hInst);
	HINSTANCE GetDllInst();
protected:
	char s_name[256];
	HINSTANCE thisDllInst;
	void * input_data;
	size_t input_length;
	void * output_data;
	size_t output_length;
};

typedef CComponent * (WINAPI * PCOMPONENT) ();
typedef CComponent * CCompnPtr;
typedef CArray<CCompnPtr, CCompnPtr&> CCompnPtrArray;


typedef struct {
	char name[36];
	char value[255];
} COMPNTPARAM;

typedef COMPNTPARAM * COMPNTPARAMPtr;

typedef struct {
	CCompnPtr   pCompnt;
	COMPNTPARAMPtr pParams[MAX_PARAMETER_NUM];
	int len;
} ACTCOMPNT;

typedef CArray<ACTCOMPNT, ACTCOMPNT&> ACTCOMPNTArray;

