/*********************************************************
Decision Engine.
This is the main class of decision engine. 
First, it loads the xml file and parses it as following:
1. create the awareness tools
2. load the reconfigurable components
3. create the compositional event and actuator for each rule.
4. start the actuator and event.
*********************************************************/

#pragma once

#define DllExport __declspec( dllexport )
#define DllImport __declspec( dllimport )
#include "ModsClient.h"
#include "ParserComm.h"
#include "AwareEvent.h"
#include "AwareSource.h"
#include "AwareActuator.h"


class DllExport CDCEngine
{
public:
	CDCEngine(void);
	CDCEngine(char *);
	void check(bool b, LPTSTR msg);
public:
	~CDCEngine(void);
public:
	enum ENGINESTATUS {NONE=0, STARTED, STOPPED};
	static void dump_com_error(_com_error &e);

	static void SetHWnd(CWnd *pWnd);
	static char * GetFormatValue();
	static void SetMetaObjectInput(void *, size_t);
	static void GetMetaObjectOutput(void **, size_t *);
	static void StartMetaObject();

	void SetXMLFile(char *);
	bool Start();
	bool Stop();

	//for test
	IXMLDOMElementPtr getDOMPtr() {return m_pDocRoot;}

protected:
	ENGINESTATUS eEngineStatus;
	bool InitializeXML(char * xmlfile);
	void StartXMLParse(IXMLDOMNodePtr pNode);
	void ParseDirectoryServer(IXMLDOMNodePtr pDS);
	void ParseAwareness(IXMLDOMNodePtr pAwares);
	void ParseSingleAware(IXMLDOMNodePtr pAware);
	void ParseComponents(IXMLDOMNodePtr pComps);
	void ParseSingleCompn(IXMLDOMNodePtr pCompn);
	void ParseRules(IXMLDOMNodePtr pRules);
	void ParseSingleRule(IXMLDOMNodePtr pRule);
	CAwareEventPtr ParseCondition(IXMLDOMNodePtr pCond);
	CAwareActuatorPtr ParseAction(IXMLDOMNodePtr pAct);
	ACTCOMPNT ParseSingleAct(IXMLDOMNodePtr pAct);
	void ParseSynAction(CAwareEvent** ppEvent, IXMLDOMNodePtr pSnyAct);

	void RegisterListener(HANDLE plis, CAwareEvent * pEvent);

private:
	char				m_xmlFile[255];
	IXMLDOMElementPtr	m_pDocRoot;
	CModsClient			* pMods;
	CAwareSrcPtrArray	m_pawareArray;
	CCompnPtrArray		m_pcompnArray;
	

	CEventPtrArray		m_peventArray;
	CActuatorPtrArray	m_pactutorArray;
	char m_cEventTag;
};
