/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
#include "AvailableBW.h"
#include "..\..\Source\IniFile.h"

void * CAvailableBW::pObject;

CAvailableBW::CAvailableBW(void) : CAwareTool()
{
	cid = CCMId("AvailableBW", 2.0);
	m_nPacketSize	= 64;
	m_nPacketNum	= 3;
	m_nInterval		= 500;
	status_socket	= DISCONNECTED;
	status_thread	= CLOSED;
	m_bAddrReady	= false;
	QueryPerformanceFrequency(&ticksPerSecond);

}

CAvailableBW::~CAvailableBW(void)
{
	Stop();
}

bool CAvailableBW::SetConfParam(CWord key, CWord value)
{
	if (CONFIG_PARAM_PACKET_SIZE == key) {
		m_nPacketSize = atoi(value.get());
	}
	else if (CONFIG_PARAM_PACKET_NUM == key) {
		m_nPacketNum = atoi(value.get());
	}
	else if (CONFIG_PARAM_INTERVAL == key) {
		m_nInterval = atoi(value.get());
	}
	else {
		return false;
	}
	return true;
}

bool CAvailableBW::SetAppParam(CWord key, void * value)
{
	if (APP_PARAM_REMOTE_ADDR == key) {
		u_long addr;
		LPHOSTENT remoteHost;
		char * ip = (char *)value;

		if (isalpha(ip[0]))   /* host address is a name */
			remoteHost = gethostbyname(ip);
		else {
			addr = inet_addr(ip);
			remoteHost = gethostbyaddr((char *)&addr, 4, AF_INET);
		}
		if (remoteHost == NULL) {
			AfxMessageBox(_T("GetHost Error: Destination Host not found!"), MB_OK, 0);
			CloseSocket();
			return false;
		}		

		memset(&m_remoteAddr,0,sizeof(m_remoteAddr));
		m_remoteAddr.sin_family = AF_INET;
		memcpy((char *)&m_remoteAddr.sin_addr.s_addr, remoteHost->h_addr_list[0],remoteHost->h_length);
		m_remoteAddr.sin_port = htons(BANDWITH_SERVER_PORT);

		m_bAddrReady = true;
		if (status_socket >= CREATED) status_socket = CONNECTED;
	}
	else if (CONFIG_FILE_PARAM_FILE_AREA == key) {
	}
	else {
		return false;
	}
	return true;
}

bool CAvailableBW::Start()
{
	if (m_bStarted) return true;

	m_sendbuf = new char[m_nPacketSize];
	StartNetwork();
	CreateSocket();
	StartTimer();
	CreateRecvThread();
	m_bStarted = true; 
	return true;
}

bool CAvailableBW::StartNetwork()
{
	WSADATA wsaData;
	WORD wVersionRequested = MAKEWORD(2,2);
	int nRet;

	// Init WinSock
	nRet = WSAStartup(wVersionRequested, &wsaData);
	if (nRet != 0) {
		AfxMessageBox(_T("\nError initializing WinSock\n"));
		return false;
	}

	// Check version
	if (wsaData.wVersion != wVersionRequested) {
		AfxMessageBox(_T("\nWinSock version not supported\n"));
		WSACleanup();
		return false;
	}

	status_socket = INITIALIZED;
	return true;
}

bool CAvailableBW::CreateSocket()
{
	if (status_socket < INITIALIZED) StartNetwork();

	m_sock = socket(AF_INET, SOCK_DGRAM, 0);
	if (m_sock == INVALID_SOCKET) {
		AfxMessageBox(_T("Socket Error: Can not create Socket!"));
		CloseNetwork();
		return false;
	}

	status_socket = CREATED;

	//bind local address to socket
	memset(&m_localAddr,0,sizeof(m_localAddr));
	m_localAddr.sin_family = AF_INET;
	m_localAddr.sin_addr.s_addr = htonl(INADDR_ANY);
	m_localAddr.sin_port = htons(BANDWITH_CLIENT_PORT);
	int nb = bind(m_sock,(struct sockaddr *)&m_localAddr,sizeof(m_localAddr));
	if (nb == SOCKET_ERROR) {
		AfxMessageBox(_T("Bind Error: Can not bind local address to Socket!"));
		CloseNetwork();
		return false;
	}

	if (m_bAddrReady) status_socket = CONNECTED;
	return true;
}

bool CAvailableBW::StartTimer()
{
	pObject = this;
	m_pBWTimer= SetTimer(NULL, NULL, m_nInterval, (TIMERPROC)MeasureBWProc);
	return true;
}

VOID CALLBACK CAvailableBW::MeasureBWProc(HWND hWnd, UINT uMsg, UINT idEvent, DWORD dwTime)
{
	CAvailableBW *pthisClass = (CAvailableBW*)pObject; // cast the void pointer
	pthisClass->MeasureBWFun(); // call non-static function
}

void CAvailableBW::MeasureBWFun()
{
	if (status_socket != CONNECTED) return;

	LARGE_INTEGER tick;   // A point in time

	for (int i=0; i<1; i++) {
		QueryPerformanceCounter(&tick);
		*(LARGE_INTEGER *)m_sendbuf = tick;
		sendto(m_sock, m_sendbuf, m_nPacketSize, 0, (struct sockaddr *)&m_remoteAddr, sizeof(m_remoteAddr));
	}
}

void CAvailableBW::StopTimer()
{
	KillTimer(NULL, m_pBWTimer);
}

void CAvailableBW::CloseSocket()
{
	if (status_socket >= CREATED) {
		shutdown(m_sock, SD_BOTH);
		closesocket(m_sock);
		status_socket = INITIALIZED;
	}
}

void CAvailableBW::CloseNetwork()
{
	if (status_socket > INITIALIZED) CloseSocket();
	if (status_socket > DISCONNECTED) {
		WSACleanup();
		status_socket = DISCONNECTED;
	}
}

bool CAvailableBW::Stop()
{
	if (!m_bStarted) return true;

	StopTimer();
	CloseSocket();
	CloseNetwork();
	CloseRecvThread();
	delete[] m_sendbuf;
	m_bStarted = false;

	return true;
}


BOOL CAvailableBW::CreateRecvThread()
{
	m_threadRecv = CreateThread(NULL, 0, ThreadRecv, this, 0, NULL);
	if (m_threadRecv == NULL) return FALSE;

	status_thread  = RUNNING;
	return TRUE;
}

VOID CAvailableBW::CloseRecvThread()
{
	int watchDog = 0;
	while (m_threadRecv)
	{
		status_thread = CLOSING;
		SetThreadPriority(m_threadRecv, THREAD_PRIORITY_HIGHEST);
		if((::WaitForSingleObject(m_threadRecv, 1000) != WAIT_TIMEOUT) ||
			watchDog >= 5)
		{
			CloseHandle(m_threadRecv);
			status_thread = CLOSED;
			m_threadRecv = NULL;
		}
		watchDog++;
	}
}

DWORD WINAPI CAvailableBW::ThreadRecv(LPVOID arg)
{
	CAvailableBW * thisClass = (CAvailableBW *)arg;
	thisClass->RecvThread();
	return 0;
}

VOID CAvailableBW::RecvThread()
{
	LARGE_INTEGER tick1, tick2;

	fd_set socks;  // select array of the socket
	//recv two packets
	struct timeval tv;  //the time of select 
	tv.tv_sec = 0;	tv.tv_usec = m_nPacketNum * m_nInterval * 1000;

	struct sockaddr recvAddr;
	int rcvAddrLen = sizeof(recvAddr);
	int rcvLen;

	double bw;

	while(status_thread == RUNNING) {
		FD_ZERO(&socks);	FD_SET(m_sock,&socks);
		int n = select(m_sock+1, &socks, NULL,NULL, &tv);

		if (n <= 0) {
			bw = 8.0; 
		}
		else {
			QueryPerformanceCounter(&tick2);
			rcvLen = recvfrom(m_sock, m_sendbuf, m_nPacketSize, 0, (struct sockaddr *)&recvAddr, &rcvAddrLen);
			tick1 = *(LARGE_INTEGER *)m_sendbuf;
			double t = double(tick2.QuadPart-tick1.QuadPart)/double(ticksPerSecond.QuadPart);

			if (t > double(tv.tv_usec)/1000000) 
				bw = 8.0;
			else 
				bw = 12.0; 
			if (status_thread != RUNNING) break;
			if (rcvLen ==SOCKET_ERROR || rcvLen <=0) break;
		}

		SetValue(&bw, sizeof(bw)); 
		NotifyConditioner();
	}

	status_thread = CLOSING;

	if(status_socket != DISCONNECTED) {
		CloseSocket();
		CloseNetwork();
	}

	// Self terminate the thread
	status_thread = CLOSED;
	m_threadRecv = NULL;
}


CAvailableBW * GetThisComponent(void)
{
	return new CAvailableBW();
}