/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
#include "Compress.h"

BYTE CCompress::m_cdata[MAX_BUFF_SIZE];
int  CCompress::m_clen;


BOOL APIENTRY DllMain( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
{
	switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH: break;
   }
   return TRUE;
}

CCompress::CCompress(void)
{
	cid = CCMId("Compress", 1.0);

	m_nCompressionRatio = 0;
	input_data		= NULL;
	input_length	= 0;
	output_data		= NULL;
	output_length	= 0;

	// Initialize the bitmapinfo
	m_bmpInfo.bmiHeader.biSize   = 40;
	m_bmpInfo.bmiHeader.biWidth  = COM_SUPPORTED_WIDTH ; 
	m_bmpInfo.bmiHeader.biHeight = COM_SUPPORTED_HEIGHT; 
	m_bmpInfo.bmiHeader.biPlanes = 1; 
	m_bmpInfo.bmiHeader.biBitCount = 24; 
	m_bmpInfo.bmiHeader.biCompression = 0;
	m_bmpInfo.bmiHeader.biSizeImage = WIDTHBYTES(m_bmpInfo.bmiHeader.biWidth*m_bmpInfo.bmiHeader.biBitCount) * m_bmpInfo.bmiHeader.biHeight; 
	m_bmpInfo.bmiHeader.biXPelsPerMeter = 0; 
	m_bmpInfo.bmiHeader.biYPelsPerMeter = 0; 
	m_bmpInfo.bmiHeader.biClrUsed = 0; 
	m_bmpInfo.bmiHeader.biClrImportant = 0; 

	// Initialize table for RGB to YUV conversion
	InitLookupTable(); 

	// Set up the callback function
	WriteByteFunction = CCompress::OwnWriteFunction;

	m_param.format = GetEncodeFormat(m_bmpInfo.bmiHeader.biWidth, m_bmpInfo.bmiHeader.biHeight);
	InitH263Encoder(&m_param); 

	output_data = malloc(MAX_BUFF_SIZE);
}

CCompress::~CCompress(void)
{
	m_nCompressionRatio = 0;
	input_data		= NULL;
	input_length	= 0;
	if (output_data	!= NULL) {
		free(output_data);
		output_data = NULL;
		output_length = 0;
	}
}

bool CCompress::SetConfParam(CWord key, CWord value)
{
	if (key == COMP_CONF_PARA_COMP_WIDTH) {
		m_bmpInfo.bmiHeader.biWidth = atoi(value.get());
		m_bmpInfo.bmiHeader.biSizeImage = WIDTHBYTES(m_bmpInfo.bmiHeader.biWidth*m_bmpInfo.bmiHeader.biBitCount) * m_bmpInfo.bmiHeader.biHeight; 
	}
	else if (key == COMP_CONF_PARA_COMP_HEIGHT) {
		m_bmpInfo.bmiHeader.biHeight = atoi(value.get());
		m_bmpInfo.bmiHeader.biSizeImage = WIDTHBYTES(m_bmpInfo.bmiHeader.biWidth*m_bmpInfo.bmiHeader.biBitCount) * m_bmpInfo.bmiHeader.biHeight; 
	}
	else return false;
	return true;
}

bool CCompress::SetAppParam(CWord key, void * value) 
{
	return true;
}

bool CCompress::Start()
{
	if (input_data == NULL || input_length <= 0) return false;

	size_t len = 0;
	BYTE type = *((LPBYTE)input_data+len);
	len += sizeof(type);

	switch(type) {
	case BMPINFO_PACK_NO_COMPRESS:
		m_bmpInfo = *((LPBITMAPINFO)((LPBYTE)input_data+len));
		len += sizeof(m_bmpInfo);
		if (m_bmpInfo.bmiHeader.biBitCount != 24) return false;
		break;
	case BMPDATA_PACK_NO_COMPRESS:
		if (m_bmpInfo.bmiHeader.biBitCount != 24) return false;
		break;
	default:
		return false;
	}

	int format = GetEncodeFormat(m_bmpInfo.bmiHeader.biWidth, m_bmpInfo.bmiHeader.biHeight);
	if (format != m_param.format) {
		m_param.format = format;
		InitH263Encoder(&m_param);
	}

	ConvertRGB2YUV(m_bmpInfo.bmiHeader.biWidth, m_bmpInfo.bmiHeader.biHeight, (LPBYTE)input_data+len, yuv);
	
	// Reset the counter
	m_clen = 0;

	m_param.format	= GetEncodeFormat(m_bmpInfo.bmiHeader.biWidth, m_bmpInfo.bmiHeader.biHeight);
	m_param.inter	= CPARAM_INTRA;  
	m_param.Q_intra= 8;
//	m_param.data	= data;		// Data in YUV format...
	m_param.data	= yuv;		// Data in YUV format...

	Bits bits;				// Various count about compression
	// this code is urgly.... actually, there is a side-effect of this function call
	// cdata and count are returned values due to this side-effect
	CompressFrame(&m_param, &bits);

	len = 0;
	if (type == BMPINFO_PACK_NO_COMPRESS) {
		type = BMPINFO_PACK_COMPRESS;
		output_length = sizeof(BYTE) + sizeof(BITMAPINFO) + m_clen;
//		if (output_data == NULL) output_data = malloc(output_length);
		*((LPBYTE)output_data+len) = type;
		len += sizeof(type);
		*((LPBITMAPINFO)((LPBYTE)output_data+len)) = m_bmpInfo;
		len += sizeof(BITMAPINFO);
	}
	else if (type == BMPDATA_PACK_NO_COMPRESS) {
		type = BMPDATA_PACK_COMPRESS;
		output_length = sizeof(BYTE) + m_clen;
//		if (output_data == NULL) output_data = malloc(output_length);
		*((LPBYTE)output_data+len) = type;
		len += sizeof(BYTE);
	}
	memcpy((LPBYTE)output_data+len, m_cdata, m_clen);

	return true;
}

bool CCompress::Stop()
{
	return true;
}

int CCompress::GetEncodeFormat(int width, int height)
{
  if (width == QCIF_YWIDTH && height == QCIF_YHEIGHT) return CPARAM_QCIF;
  else if (width == CIF_YWIDTH && height == CIF_YHEIGHT) return CPARAM_CIF;
  else if (width == SQCIF_YWIDTH && height == SQCIF_YHEIGHT) return CPARAM_SQCIF;
  else if (width == CIF4_YWIDTH && height == CIF4_YHEIGHT) return CPARAM_4CIF;
  else if (width == CIF16_YWIDTH && height == CIF16_YHEIGHT) return CPARAM_16CIF;
  return CPARAM_OTHER;
}

void CCompress::OwnWriteFunction(int byte)
{
	if(m_clen < MAX_BUFF_SIZE) 
		m_cdata[m_clen++]=(unsigned char)byte;
}

CCompress * GetThisComponent(void)
{
	return new CCompress();
}