/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
#include "Decompress.h"
#include "encoder/libr263.h"
#include "decoder/tmndec.h"

BOOL APIENTRY DllMain( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
{
	switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH: break;
   }
   return TRUE;
}

CDecompress::CDecompress(void)
{
	cid = CCMId("Decompress", 1.0);

	input_data		= NULL;
	input_length	= 0;
	output_data		= NULL;
	output_length	= 0;

	// Initialize the bitmapinfo
	m_bmpInfo.bmiHeader.biSize   = 40;
	m_bmpInfo.bmiHeader.biWidth  = COM_SUPPORTED_WIDTH ; 
	m_bmpInfo.bmiHeader.biHeight = COM_SUPPORTED_HEIGHT; 
	m_bmpInfo.bmiHeader.biPlanes = 1; 
	m_bmpInfo.bmiHeader.biBitCount = 24; 
	m_bmpInfo.bmiHeader.biCompression = 0;
	m_bmpInfo.bmiHeader.biSizeImage = WIDTHBYTES(m_bmpInfo.bmiHeader.biWidth*m_bmpInfo.bmiHeader.biBitCount) * m_bmpInfo.bmiHeader.biHeight; 
	m_bmpInfo.bmiHeader.biXPelsPerMeter = 0; 
	m_bmpInfo.bmiHeader.biYPelsPerMeter = 0; 
	m_bmpInfo.bmiHeader.biClrUsed = 0; 
	m_bmpInfo.bmiHeader.biClrImportant = 0; 

	// Initialize table for RGB to YUV conversion
	InitLookupTable(); 
	//Initialize decompressor
	InitH263Decoder();

	output_data = malloc(MAX_BUFF_SIZE);
}

CDecompress::~CDecompress(void)
{
	input_data		= NULL;
	input_length	= 0;
	if (output_data	!= NULL) {
		free(output_data);
		output_data = NULL;
		output_length = 0;
	}
}

bool CDecompress::SetConfParam(CWord key, CWord value)
{
	if (key == DECO_CONF_PARA_COMP_WIDTH) {
		m_bmpInfo.bmiHeader.biWidth = atoi(value.get());
		m_bmpInfo.bmiHeader.biSizeImage = WIDTHBYTES(m_bmpInfo.bmiHeader.biWidth*m_bmpInfo.bmiHeader.biBitCount) * m_bmpInfo.bmiHeader.biHeight; 
	}
	else if (key == DECO_CONF_PARA_COMP_HEIGHT) {
		m_bmpInfo.bmiHeader.biHeight = atoi(value.get());
		m_bmpInfo.bmiHeader.biSizeImage = WIDTHBYTES(m_bmpInfo.bmiHeader.biWidth*m_bmpInfo.bmiHeader.biBitCount) * m_bmpInfo.bmiHeader.biHeight; 
	}
	else return false;
	return true;
}

bool CDecompress::SetAppParam(CWord key, void * value) 
{
	return true;
}

bool CDecompress::Start()
{
	if (input_data == NULL || input_length <= 0) return false;

	size_t len = 0;
	BYTE type = *((LPBYTE)input_data+len);
	len += sizeof(type);

	switch(type) {
	case BMPINFO_PACK_COMPRESS:
		m_bmpInfo = *((LPBITMAPINFO)((LPBYTE)input_data+len));
		len += sizeof(m_bmpInfo);
		if (m_bmpInfo.bmiHeader.biBitCount != 24) return false;

		output_length = sizeof(BYTE) + sizeof(BITMAPINFO) + m_bmpInfo.bmiHeader.biSizeImage;
//		if (output_data == NULL) output_data = malloc(output_length);
		type = BMPINFO_PACK_NO_COMPRESS;
		*((LPBYTE)output_data) = type;
		*((LPBITMAPINFO)((LPBYTE)output_data + sizeof(BYTE))) = m_bmpInfo;
		break;
	case BMPDATA_PACK_COMPRESS:
		if (m_bmpInfo.bmiHeader.biBitCount != 24) return false;

		output_length = sizeof(BYTE) + m_bmpInfo.bmiHeader.biSizeImage;
//		if (output_data == NULL) output_data = malloc(output_length);
		type = BMPDATA_PACK_NO_COMPRESS;
		*((LPBYTE)output_data) = type;
		break;
	default:
		return false;
	}

	int retvalue =DecompressFrame((LPBYTE)input_data+len,input_length-len, (LPBYTE)output_data+len, output_length-len);
	if (!retvalue)	return false;

	return true;
}

bool CDecompress::Stop()
{
	return true;
}

CDecompress * GetThisComponent(void)
{
	return new CDecompress();
}
