/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
#include <afxwin.h>
#include "Grab.h"

CCriticalSection CGrab::m_hGrabLock;

BOOL APIENTRY DllMain( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
{
	switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH: break;
   }
   return TRUE;
}

CGrab::CGrab(void)
{
	cid = CCMId("Grab", 1.0);

	input_data		= NULL;
	input_length	= 0;
	output_data		= NULL;
	output_length	= 0;

	videoState		= NONE;
	hMainWnd		= NULL;
	hCapWnd			= NULL;
	nDispWidth		= DEFAULT_DISPLAY_WIDTH;
	nDispHeight		= DEFAULT_DISPLAY_HEIGHT;
	m_bAutoCapture	= false;
	m_bPrviewMode	= false;
	m_bLocalShow	= true;
	m_bSaveMode		= true;
	
	// Initialize the bitmapinfo
	m_bmpInfo.bmiHeader.biSize   = 40;
	m_bmpInfo.bmiHeader.biWidth  = CAP_ACCEPTABLE_WIDTH ; 
	m_bmpInfo.bmiHeader.biHeight = CAP_ACCEPTABLE_HEIGHT; 
	m_bmpInfo.bmiHeader.biPlanes = 1; 
	m_bmpInfo.bmiHeader.biBitCount = 24; 
	m_bmpInfo.bmiHeader.biCompression = 0;
	m_bmpInfo.bmiHeader.biSizeImage = WIDTHBYTES(m_bmpInfo.bmiHeader.biWidth*m_bmpInfo.bmiHeader.biBitCount) * m_bmpInfo.bmiHeader.biHeight; 
	m_bmpInfo.bmiHeader.biXPelsPerMeter = 0; 
	m_bmpInfo.bmiHeader.biYPelsPerMeter = 0; 
	m_bmpInfo.bmiHeader.biClrUsed = 0; 
	m_bmpInfo.bmiHeader.biClrImportant = 0; 

	m_StretchInfo.bmiHeader.biSize   = 40;
	m_StretchInfo.bmiHeader.biWidth  = COM_SUPPORTED_WIDTH; 
	m_StretchInfo.bmiHeader.biHeight = COM_SUPPORTED_HEIGHT; 
	m_StretchInfo.bmiHeader.biPlanes = 1; 
	m_StretchInfo.bmiHeader.biBitCount = 24; 
	m_StretchInfo.bmiHeader.biCompression = 0;
	m_StretchInfo.bmiHeader.biSizeImage = WIDTHBYTES(m_StretchInfo.bmiHeader.biWidth*m_StretchInfo.bmiHeader.biBitCount) * m_StretchInfo.bmiHeader.biHeight; 
	m_StretchInfo.bmiHeader.biXPelsPerMeter = 0; 
	m_StretchInfo.bmiHeader.biYPelsPerMeter = 0; 
	m_StretchInfo.bmiHeader.biClrUsed = 0; 
	m_StretchInfo.bmiHeader.biClrImportant = 0; 

	// Create the event;
	m_hEvent = CreateEventA(NULL,FALSE,FALSE,"ResumeStart");

	output_data = malloc(MAX_BUFF_SIZE);
}

CGrab::~CGrab(void)
{
	Stop();
	input_data		= NULL;
	input_length	= 0;
	if (output_data	!= NULL) {
		free(output_data);
		output_data = NULL;
		output_length = 0;
	}
}

bool CGrab::SetConfParam(CWord key, CWord value)
{
	if (key == GRAB_CONF_PARA_DISP_WIDTH) {
		nDispWidth = atoi(value.get());
	}
	else if (key == GRAB_CONF_PARA_DISP_HEIGHT) {
		nDispHeight = atoi(value.get());
	}
	else if (key == GRAB_CONF_PARA_CAPT_WIDTH) {
		m_bmpInfo.bmiHeader.biWidth = atoi(value.get());
		m_bmpInfo.bmiHeader.biSizeImage = WIDTHBYTES(m_bmpInfo.bmiHeader.biWidth*m_bmpInfo.bmiHeader.biBitCount) * m_bmpInfo.bmiHeader.biHeight; 
	}
	else if (key == GRAB_CONF_PARA_CAPT_HEIGHT) {
		m_bmpInfo.bmiHeader.biHeight = atoi(value.get());
		m_bmpInfo.bmiHeader.biSizeImage = WIDTHBYTES(m_bmpInfo.bmiHeader.biWidth*m_bmpInfo.bmiHeader.biBitCount) * m_bmpInfo.bmiHeader.biHeight; 
	}
	else if (key == GRAB_CONF_PARA_COMP_WIDTH) {
		m_StretchInfo.bmiHeader.biWidth = atoi(value.get());
		m_StretchInfo.bmiHeader.biSizeImage = WIDTHBYTES(m_StretchInfo.bmiHeader.biWidth*m_StretchInfo.bmiHeader.biBitCount) * m_StretchInfo.bmiHeader.biHeight; 
	}
	else if (key == GRAB_CONF_PARA_COMP_HEIGHT) {
		m_StretchInfo.bmiHeader.biHeight = atoi(value.get());
		m_StretchInfo.bmiHeader.biSizeImage = WIDTHBYTES(m_StretchInfo.bmiHeader.biWidth*m_StretchInfo.bmiHeader.biBitCount) * m_StretchInfo.bmiHeader.biHeight; 
	}
	else return false;
	return true;
}

bool CGrab::SetAppParam(CWord key, void * value)
{
	if (key == GRAB_APP_PARA_MAINWND) {
		hMainWnd	= *((HWND *)value);
	}
	else if (key == GRAB_APP_PARA_DISPWIDTH) {
		nDispWidth	= *((int *)value);
	}
	else if (key == GRAB_APP_PARA_DISPHEIGHT) {
		nDispHeight	= *((int *)value);
	}	
	else if (key == GRAB_APP_PARA_AUTOCAPTURE) {
		SetAutoCapture(*((bool *)value));
	}
	else if (key == GRAB_APP_PARA_SHOWLOCAL) {
		SetLocalShow(*((bool *)value));
	}
	else if (key == GRAB_APP_PARA_SAVEMODE) {
		SetSaveMode(*((bool *)value));
	}
	else if (key == GRAB_APP_PARA_VIDEOSOURCE) {
		SetVideoSource();
	}
	else if (key == GRAB_APP_PARA_VIDEOFORMAT) {
		SetVideoFormat();
	}
	else if (key == GRAB_APP_PARA_VIDEODISPLAY) {
		SetVideoDisplay();
	}
	else return false;
	return true;
}

void CGrab::SetAutoCapture(bool b)
{
	m_bAutoCapture = b;
}

void CGrab::SetLocalShow(bool b)
{
	m_bLocalShow = b;
}

void CGrab::SetSaveMode(bool b)
{
	m_bSaveMode = b;
}

void CGrab::SetVideoSource()
{
	if (videoState < CONNECTED) return;
	if(m_gCapDriverCaps.fHasDlgVideoSource)	capDlgVideoSource(hCapWnd); //Video source dialog
}

void CGrab::SetVideoFormat()
{
	if (videoState < CONNECTED) return;
	if(m_gCapDriverCaps.fHasDlgVideoFormat)	capDlgVideoFormat(hCapWnd); // Video format dialog
}

void CGrab::SetVideoDisplay()
{
	if (videoState < CONNECTED) return;
	if(m_gCapDriverCaps.fHasDlgVideoDisplay) capDlgVideoDisplay(hCapWnd); // Video display dialog
}

void CGrab::ResizeGrabWnd(int cx, int cy)
{
	nDispWidth	= cx;
	nDispHeight	= cy;
	SetWindowPos(hCapWnd,NULL,0,0,cx,cy,SWP_NOZORDER|SWP_NOMOVE);
}

bool CGrab::capSetCaptureParam()
{
	CAPTUREPARMS capParms;
	capCaptureGetSetup(hCapWnd, &m_OldcapParms,sizeof(capParms));
	capParms = m_OldcapParms;
	capParms.fAbortLeftMouse	= FALSE;
	capParms.fAbortRightMouse	= FALSE;
	capParms.fYield				= TRUE;
	capParms.fCaptureAudio		= FALSE;
	capParms.wPercentDropForError= 50;
//	CapParms.dwRequestMicroSecPerFrame= 66667;

	if(capCaptureSetSetup(hCapWnd,&capParms,sizeof(capParms))==FALSE) return false;
	return true;
}

bool CGrab::capSetVideoParam()
{
	// Set Video Format 
	capGetVideoFormat(hCapWnd,&m_OldbmpInfo,sizeof(m_OldbmpInfo));
/*	m_bmpInfo = m_OldbmpInfo;
	m_bmpInfo.bmiHeader.biWidth  = CAP_ACCEPTABLE_WIDTH;
	m_bmpInfo.bmiHeader.biHeight = CAP_ACCEPTABLE_HEIGHT;
*/
	if (capSetVideoFormat(hCapWnd,&m_bmpInfo,sizeof(m_bmpInfo))==FALSE) return false;
	return true;
}

bool CGrab::InitializeVideo()
{
	if (hMainWnd == NULL) return false;

	hCapWnd = capCreateCaptureWindow ( _T("Capture Window"), 
		WS_CHILD /* | WS_VISIBLE */,
		0, 0, nDispWidth, nDispHeight, 
		hMainWnd, (int)0);

	if(hCapWnd==NULL) return false;
	videoState = INITIALIZED;
	return true;
}

bool CGrab::ConnectVideo()
{
	if (videoState < INITIALIZED) if (!InitializeVideo()) return false;

	if (capSetUserData(hCapWnd,this)==FALSE) return false;

	if (capSetCallbackOnError(hCapWnd, ErrorCallbackProc)==FALSE) return false;

	if (capSetCallbackOnStatus(hCapWnd, StatusCallbackProc)==FALSE) return false;

	if (capSetCallbackOnVideoStream(hCapWnd,OnCaptureVideo)==FALSE)	return false; 

	if (capSetCallbackOnFrame(hCapWnd, OnGrabFrame)==FALSE) return false;

	if (capDriverConnect(hCapWnd, 0)==FALSE) return false;

	capDriverGetCaps(hCapWnd,&m_gCapDriverCaps,sizeof(CAPDRIVERCAPS));
	capSetCaptureParam();
	capSetVideoParam();

	videoState = CONNECTED;
	return true;
}

bool CGrab::PreviewVideo()
{
	if (videoState < CONNECTED) return false;

	CWnd *pWnd = CWnd::FromHandle(hCapWnd);
	pWnd->ShowWindow(TRUE);
	if (capPreviewRate(hCapWnd, 66)==FALSE) return false;	// set the rate of Preview mode
	if (capPreviewScale(hCapWnd, TRUE)==FALSE) return false;// set the preview scale can be stretched.
	if (capPreview(hCapWnd, TRUE)==FALSE) return false;		// start the Preview mode 
	if (m_gCapDriverCaps.fHasOverlay) capOverlay(hCapWnd,TRUE);	// start Overlay mode

	m_bPrviewMode = true;
	return true;
}

bool CGrab::Start()
{
//	if (videoState == STARTED) return true;
	if (videoState < CONNECTED) {
		if (!ConnectVideo()) return false;
	}
	
	if (m_bAutoCapture) {
		if (videoState < STARTED) capCaptureSequenceNoFile(hCapWnd);
	}
	else {
		capGrabFrame(hCapWnd);
		WaitForSingleObject(m_hEvent,INFINITE);
	}
	videoState = STARTED;

	return true;
}

bool CGrab::Stop()
{
	BOOL ret;
	if (videoState >= CONNECTED) {
		CloseHandle(m_hEvent);

		ret = capCaptureAbort(hCapWnd);
		TRACE1("abort %d\n", (int)ret);
		Sleep(500);

		// Disable the callback function..
		ret = capSetCallbackOnStatus(hCapWnd,NULL);
		TRACE1("status %d\n", (int)ret);
		ret = capSetCallbackOnError(hCapWnd,NULL);
		TRACE1("error %d\n", (int)ret);

		// How to stop the video stream is a problem;
		ret = capSetCallbackOnVideoStream(hCapWnd, NULL); 
		TRACE1("videostream %d\n", (int)ret);
		ret = capSetCallbackOnFrame(hCapWnd, NULL);
		TRACE1("frame %d\n", (int)ret);
		ret = capPreview(hCapWnd, FALSE);
		TRACE1("preview %d\n", (int)ret);

		capCaptureSetSetup(hCapWnd,&m_OldcapParms,sizeof(m_OldcapParms));
		capSetVideoFormat(hCapWnd,&m_OldbmpInfo,sizeof(m_OldbmpInfo));

		// Finally disconnect the driver
//		if (capDriverDisconnect(hCapWnd) == FALSE) return false;
		ret = capDriverDisconnect(hCapWnd);
		TRACE1("disconnect %d\n", (int)ret);

		videoState = DISCONNECTED;
	}
	return true;
}

void CGrab::OnReceiveFrame(LPVIDEOHDR lpheader)
{
	if (m_bLocalShow) {
//		SetDIBitsToDevice(GetDC(hMainWnd), 0, 0, nDispWidth, nDispHeight, 
//			0, 0, 0, nCapHeight, lpheader->lpData, &m_bmpInfo, DIB_RGB_COLORS); 
		HDC hTargetDC = GetDC(hMainWnd);
		SetStretchBltMode(hTargetDC, COLORONCOLOR ); 
		StretchDIBits(hTargetDC, 0, 0, nDispWidth, nDispHeight, 0, 0, m_bmpInfo.bmiHeader.biWidth, m_bmpInfo.bmiHeader.biHeight, 
			lpheader->lpData, &m_bmpInfo, DIB_RGB_COLORS, SRCCOPY); 
		ReleaseDC(hMainWnd, hTargetDC);
	}


	BYTE type;
	size_t len = 0;
	
	if (videoState < STARTED || !m_bSaveMode) {
		type = BMPINFO_PACK_NO_COMPRESS;
		*((LPBYTE)output_data+len) = type;
		len += sizeof(type);
		*((LPBITMAPINFO)((LPBYTE)output_data+len)) = m_StretchInfo;
		len += sizeof(m_bmpInfo);
		output_length	= sizeof(BYTE) + sizeof(BITMAPINFO) + m_StretchInfo.bmiHeader.biSizeImage;
	}
	else {
		BYTE type = BMPDATA_PACK_NO_COMPRESS;
		memcpy((LPBYTE)output_data+len, &type, sizeof(type));
		len += sizeof(type);
		output_length	= sizeof(BYTE) + m_StretchInfo.bmiHeader.biSizeImage;
	}

	if (m_bmpInfo.bmiHeader.biWidth != m_StretchInfo.bmiHeader.biWidth 
		|| m_bmpInfo.bmiHeader.biHeight != m_StretchInfo.bmiHeader.biHeight)
		StretchToSupportedBmp(lpheader, (LPBYTE)output_data+len);
	else memcpy((LPBYTE)output_data+len, lpheader->lpData, lpheader->dwBufferLength);

	SetEvent(m_hEvent);
}

void CGrab::StretchToSupportedBmp(LPVIDEOHDR lpheader, LPBYTE lpOut)
{
	LPBYTE	lpTargetBits;

	// Gonna use DIBSections and BitBlt() to do the conversion, so make 'em 
	HDC hDC			= GetDC( NULL );
	HDC hTargetDC	= CreateCompatibleDC( hDC ); 
	HBITMAP hTargetBitmap	= CreateDIBSection( hDC, &m_StretchInfo, DIB_RGB_COLORS, (VOID **)&lpTargetBits, NULL, 0 ); 
	HBITMAP hOldTargetBitmap= (HBITMAP)SelectObject( hTargetDC, hTargetBitmap ); 
 
	SetStretchBltMode( hTargetDC, COLORONCOLOR ); 
	StretchDIBits( hTargetDC, 0, 0, m_StretchInfo.bmiHeader.biWidth, m_StretchInfo.bmiHeader.biHeight, 0, 0, 
		m_bmpInfo.bmiHeader.biWidth, m_bmpInfo.bmiHeader.biHeight, lpheader->lpData, &m_bmpInfo, DIB_RGB_COLORS, SRCCOPY); 
 
    // Clean up and delete the DCs 
    SelectObject( hTargetDC, hOldTargetBitmap ); 
    DeleteDC( hTargetDC ); 
    ReleaseDC( NULL, hDC );

	memcpy(lpOut, lpTargetBits, m_StretchInfo.bmiHeader.biSizeImage);
    DeleteObject( hTargetBitmap );
}

LRESULT CALLBACK OnCaptureVideo(HWND hWnd, LPVIDEOHDR lpheader)
{
	static int ntimes = 0;
	TRACE1("video times = %d\n", ntimes++);

	CGrab *grab=(CGrab *)capGetUserData(hWnd);
	if (grab == NULL) return FALSE;
	if (grab->videoState < CGrab::CONNECTED) return FALSE;
	if (grab->m_bPrviewMode) return FALSE;

	grab->OnReceiveFrame(lpheader);
	return TRUE;
}

LRESULT CALLBACK OnGrabFrame(HWND hWnd, LPVIDEOHDR lpheader)
{
	static int ntimes = 0;
	TRACE1("frame times = %d\n", ntimes++);

	CGrab *grab=(CGrab *)capGetUserData(hWnd);
	if (grab == NULL) return FALSE;
	if (grab->videoState < CGrab::CONNECTED) return FALSE;
	if (grab->m_bPrviewMode) return FALSE;

	grab->OnReceiveFrame(lpheader);
	return TRUE;
}

LRESULT CALLBACK StatusCallbackProc(HWND hWnd,int nID,LPSTR lpStatusText)
{
/*	CGrab *grab=(CGrab *)capGetUserData(hWnd);
	if (grab == NULL) return FALSE;
	if (grab->videoState < CGrab::CONNECTED) return false;

	CAPSTATUS gCapStatus;
	capGetStatus(hWnd,&gCapStatus,sizeof(CAPSTATUS));
	SetWindowPos(grab->hCapWnd,NULL,0,0,gCapStatus.uiImageWidth,gCapStatus.uiImageHeight,SWP_NOZORDER|SWP_NOMOVE);
	if(nID==0){
		SetWindowText(grab->hCapWnd, _T("hello"));
		return (LRESULT)TRUE;
	}
	CString gachBuffer;
	gachBuffer.Format(_T("Status# %d: %s"),nID,lpStatusText);
	SetWindowText(grab->hCapWnd,gachBuffer);
*/	return (LRESULT)TRUE;
}

LRESULT CALLBACK ErrorCallbackProc(HWND hWnd,int nErrID,LPSTR lpErrorText)
{
/*	CSingleLock csl(&CGrab::m_hGrabLock);
	csl.Lock();	//ENTER CRITICAL REGION

	CGrab *grab=(CGrab *)capGetUserData(hWnd);
	if (grab == NULL) return FALSE;
	if(grab->hCapWnd == NULL) return FALSE;

	if(nErrID==0) return TRUE;
	CString gachBuffer;
	gachBuffer.Format(_T("Error# %d"),nErrID);
	CString gerrText;
	gerrText = lpErrorText;
	MessageBox(grab->hMainWnd, gerrText, gachBuffer, MB_OK | MB_ICONEXCLAMATION);

	csl.Unlock();//EXIT CRITICAL REGION
*/	return TRUE;
}

CGrab * GetThisComponent(void)
{
	return new CGrab();
}