/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
/*********************************************************
Actuator
Actuator is the action of the application when one rule 
is sufficed and notified by the compositional event.
There is one actuator in each rule, corresponding to one
compositional event. 
When the rule is sufficed, the corresponding actuator will
update the architecture (metaobject)
*********************************************************/
#pragma once

#define THREAD_TERMINATION_MAXWAIT	3


#include "ReconfigComponent.h"

class CAwareActuator
{
	friend class CAwareMetaObject;
public:
	CAwareActuator(void);
public:
	virtual ~CAwareActuator(void);

public:
	void Active();
	void SetInput(void *, size_t len);
	void GetOutput(void ** data, size_t * len);
	void Start();
	void Process(void *, size_t, void ** out = NULL, size_t * olen = NULL);
	void Stop();
	void AddCompt(CRecfnCompntPtr);

	void format(char *buf);

protected:
	void *	act_input;
	size_t	input_len;
	void *	act_output;
	size_t	output_len;
	CRecfnCompntPtrArray m_pConfCompnArray;

// for active actuator
public:
	void RequireSync(bool b = true);
	bool CheckSyncReq();
	void SetSyncIndex(BYTE m);
	BYTE GetSyncIndex();
protected:
	bool	m_bSyncRequired;
	BYTE	m_syncIndex;


// for synchronization actuator
public:
protected:

// for reactive actuator
public:
	char * ToString();
	void FromString(char * inStr);
	void SetRemoteAddr(unsigned long addr);
	unsigned long GetRemoteAddr();
protected:
	char *	outString;
	unsigned long m_remoteAddr;
};

typedef CAwareActuator * CActuatorPtr;
typedef CArray<CActuatorPtr, CActuatorPtr&> CActuatorPtrArray;