/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
/***********************************************************
CAwareMetaObject
This class is the architecture of the configured components.
Because there is only one architecture, it is a global variable.
It gets the components data from acutuator that is notified by
the compositional event.
To form the architecture, first, all the parameters of each 
component in the acutuator must be reset because the components 
are also global variables and only has one copy of each component
Currently, there is no synchronization part in this architecture.
************************************************************/

#pragma once

#include <afxmt.h>	// for CCriticalSection
#include <Afxsock.h> // for socket

#include "AwareEvent.h"
#include "AwareActuator.h"
#include "AwareSource.h"
#include "ReconfigComponent.h"

static const char * szActionDoneEvent = "ActionHasBeenDone";
static const char * szReceivDoneEvent = "ReceivHasBeenDone";
static const char * szArchChangeEvent = "ArchitectureHasBeenUpdated";

class CAwareMetaObject
{
	friend class CAwareXMLParser;
public:
	CAwareMetaObject(void);
public:
	~CAwareMetaObject(void);

public:
	static CCriticalSection m_dataLock, m_syncLock;
	enum THREADSTATUS {CLOSED=0, CLOSING, RUNNING};
	enum SYNCSTATUS {NONE=0, DISCONNECTED, INITIALIZED, CREATED, REQUESTED, CONNECTED};

	void SetHWnd(CWnd *pWnd);
	void SetCompntAppParam(CWord, CWord, void*);
	void SetInput(void * data, size_t len);
	void GetOutput(void ** data, size_t * len);
	void Process();
	void ProcessActv(void *, size_t, void ** out = NULL, size_t * olen = NULL);
	void ProcessRecv(sockaddr *, void *, size_t, void ** out = NULL, size_t * olen = NULL);

	void AddObject(CEventPtr, CActuatorPtr, CActuatorPtr);
	void Construct();
	void Deconstruct();

	void SyncTo(char * addr);
	void Start();
	void Stop();

	void FormatOutput(int type);
	void GetFormatV(char *buf);

protected:
	static DWORD WINAPI EventWaitingThread(LPVOID arg);
	void EventWaitingFun();
	void StartWaitingThread();
	void StopWaitingThread();

	void CloseEventHandles();
	void ReleaseArchs();
	void ReleaseCompnts();

protected: // network for synchronization
	void InitReadySync();
	void CloseSyncPart();
	static DWORD WINAPI SyncReceivingThread(LPVOID arg);
	void SyncReceivingFun();
	void StartReceivingThread();
	void StopReceivingThread();

	bool StartNetwork();
	bool CreateSocket();
	void CloseSocket();
	void CloseNetwork();

protected:
	void ProcBuf(char *, int, sockaddr *);
	void RequestSync(sockaddr * paddr);
	void ToSyncReqestInfo(char * buf, size_t * len);
	void ToSyncResponseInfo(char * buf, size_t * len);
	void FromSyncReqestInfo(sockaddr *, char *, size_t);
	void FromSyncResponseInfo(sockaddr *, char * , size_t);

	void SetActuatorCompnt(CActuatorPtr,CRecfnCompntPtrArray *);
	void RemoveReactiveArchs(sockaddr_in *);

	virtual void OnReceive(BYTE active = 0);	// active = 0 active message process done
												// active = 1 received message process done

private:
	void *	act_input;
	size_t	input_len;
	void *	act_output;
	size_t	output_len;

	void *	proc_output;
	size_t	proc_outlen;

	HWND	act_hWnd;
	int		m_actvIndex, m_recvIndex;

	HANDLE	*m_hSourceEvent;
	HANDLE	m_threadWaiting;
	THREADSTATUS	m_statusWaitingThread;

	// socket
	SOCKET			m_sock;
	sockaddr_in		m_localAddr, m_remoteAddr;
	HANDLE			m_threadReceiving;
	THREADSTATUS	m_statusReceivingThread;

	SYNCSTATUS		m_statusSync;

	CAwareSrcPtrArray 		m_pAwareTools;
	CRecfnCompntPtrArray 	m_pActiveAwarelets, m_pReactiveAwarelets;

	CEventPtrArray		m_pEventArray;
	CActuatorPtrArray	m_pActArray,	// process the local data and/or send the processed data out
						m_pReActArray,	// actuator constructed to process the received data
						m_pSyncActArray;// no really components in this array, used for synchronization

private:
	char fm_out[1024];
};

typedef CAwareMetaObject * CMetaObjectPtr;