/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
/*********************************************************
Component.
This is the parent class of all MobiWare components.
Any one can implement his(her) own components by inheriting
this class. 
The implemented components could be a DLL, so that our
middleware can load the dll dynamically, just like awareness
measurement tools
There is only one instance of each component in our middleware.
All the actuators or metaobject just get a pointer pointing to
the component.
*********************************************************/
#ifndef _COMPONENT_INTERFACE_
#define _COMPONENT_INTERFACE_

#define DllExport __declspec( dllexport )
#define DllImport __declspec( dllimport )

#include "common.h"
#include <afxwin.h>

class CComponent
{
public:
	CComponent(void) {}
public:
	virtual ~CComponent(void) {}
public:
	CCMId  GetCId() { return cid; }
	char * GetName() {return cid.GetName();}
	float  GetVersion() {return cid.GetVersion();}

	void SetDllInst(HINSTANCE hInst) { thisDllInst = hInst; }
	HINSTANCE GetDllInst() { return thisDllInst; }

	bool SetConfParam(COMPNTPARAM param) { return SetConfParam(param.key, param.value);	}
	virtual bool SetConfParam(CWord key, CWord value) { return false; }
	virtual bool SetAppParam(CWord key, void * value) { return false; }
	virtual bool Start() { return false; }
	virtual bool Stop() { return false; }

protected:
	CCMId cid;

	HINSTANCE thisDllInst;
};

typedef CComponent * (WINAPI * PCOMPONENT) ();
typedef CComponent * CCompnPtr;

#endif  //_COMPONENT_INTERFACE_
