/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
/*********************************************************
Awareness.
This is the parent class of all awareness measurement tools.
Any one can implement his(her) own awareness by inheriting
this class. 
The implemented awareness tools could be a DLL, so that our
middleware can load the dll dynamically, just like components
There is only one instance of each awareness tool in our 
middleware. However, this measurement tool can notify multiple
conditioner.
*********************************************************/
#pragma once


#define MAX_VALUE_SIZE		1024
#define MAX_NOTIFIER_SIZE	10

#define DATA_SOURCE_DS_SERVER	"DirectoryServer"
#define DATA_SOURCE_SHARED_TOOL	"SharedTool"
#define DATA_SOURCE_INDIV_TOOL	"IndividualTool"
#define DATA_SOURCE_CONFIG_FILE	"File"

#define CONFIG_FILE_PARAM_FILE_DIR	"fileDir"
#define CONFIG_FILE_PARAM_FILE_AREA	"fileArea"
#define CONFIG_FILE_LENGTH_NAME		"Len"
#define CONFIG_FILE_TIME_PREFIX		"t"
#define CONFIG_FILE_VALUE_PREFIX	"v"

#define APPLICATIONAWARE	"Awareness.ApplicationAware"
#define AVAILABLEBANDWIDTH	"Awareness.NetworkAware.AvailableBW"
#define	AVAILABLECPU		"Awareness.DeviceAware.AvailableCPU"


#include <afxwin.h>
#include "component.h"

class CAwareTool : public CComponent
{
public:
	CAwareTool(void) {
		m_bStarted		= false;
		m_bDataReady	= false;
		m_nListener		= 0;
	}
public:
	~CAwareTool(void) {}

public:
	bool GetValue(void *value, int *len) {
		if (m_bDataReady) {
			memcpy(value, m_chValue, m_nLength);
			*len = m_nLength;
		}
		return m_bDataReady;
	}

	void AddNotifier(HANDLE hNotifer) {
		if (m_nListener >= MAX_NOTIFIER_SIZE) return;
		m_hListeners[m_nListener++]=hNotifer;
	}
	bool IsStarted() {return m_bStarted;}

protected:
	bool SetValue(void *value, int len) {
		if (len > MAX_VALUE_SIZE)
			m_bDataReady = false;
		else {
			m_nLength = len;
			memcpy(m_chValue, value, len);
			m_bDataReady = true;
		}
		return m_bDataReady;
	}

	void NotifyConditioner() {
		for(int i=0; i<m_nListener; i++) 
			SetEvent(m_hListeners[i]);
	}

protected:
	char m_chValue[MAX_VALUE_SIZE];
	int  m_nLength;
	bool m_bDataReady;
	bool m_bStarted;

	HANDLE m_hListeners[MAX_NOTIFIER_SIZE];
	int	 m_nListener;
};

typedef CAwareTool * (WINAPI * PAWARETOOL) ();
typedef CAwareTool * CAwareToolPtr;