/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
/**************************************************************************
 * btree.h
 * This file is copied from Prof. Edwin Kay
 *
 * This file is used to defile the template class binary tree
 *************************************************************************/

#ifndef BTREE_H
#define BTREE_H
#include <stdlib.h>

template <class KEYTYPE, class DATATYPE>
class BinTree
{private:
  class Node
  {public:
    Node(const KEYTYPE &k,const DATATYPE& dat,Node *lft=NULL,Node *rt=NULL);
    KEYTYPE key;
    DATATYPE data;
    Node *child[2];
  };
 public:
  static const int LEFT=0,RIGHT=1;
  BinTree();
  BinTree(const BinTree&b);
  ~BinTree();
  void add(const KEYTYPE &k);
  void add(const KEYTYPE &k,const DATATYPE & d);
  void iteratorReset();
  KEYTYPE nextKey();
  int size()const;
  bool atEnd()const;
  bool has(const KEYTYPE& k)const;
  BinTree & operator +=(const KEYTYPE &k);
  DATATYPE & operator[](const KEYTYPE & index);
  DATATYPE operator[](const KEYTYPE & index)const;
  BinTree  operator=(const BinTree&b);
  void remove(const KEYTYPE &k);
  int countLarger(const KEYTYPE &k)const;
  int countSmaller(const KEYTYPE &k)const;
  bool sameTree(const BinTree &b);
private:
  Node *root,*keyLoc;
  static void destroy(Node *&rt);
  static int size(Node *rt);
  void add(const KEYTYPE &k,const DATATYPE & x, Node *&rt);
  static void copy(Node * oldRt,Node*&newRt);
  double & getEntry(const KEYTYPE &k,Node * rt);
  bool has(const KEYTYPE &k,Node *rt)const;
  static void check(bool b, char *mess);
  static bool sameTree(Node *a, Node *b);
  void remove(const KEYTYPE &k,Node*&rt);
  int countLarger(const KEYTYPE & k,Node *rt)const;
  int countSmaller(const KEYTYPE& k,Node *rt)const;
};



template <class KEYTYPE,class DATATYPE>
BinTree<KEYTYPE,DATATYPE>::Node::Node(
	 const KEYTYPE &k,const DATATYPE & dat,Node *lft,Node *rt)
{key=k;
 data=dat;
 child[LEFT]=lft;
 child[RIGHT]=rt;
}

template <class KEYTYPE,class DATATYPE>
BinTree<KEYTYPE,DATATYPE>:: BinTree():root(NULL){}

template <class KEYTYPE,class DATATYPE>
BinTree<KEYTYPE,DATATYPE>::BinTree(const BinTree&b)
{copy(b.root,root);}

template <class KEYTYPE,class DATATYPE>
BinTree<KEYTYPE,DATATYPE>::~BinTree()
{destroy(root);}

template <class KEYTYPE,class DATATYPE>
void BinTree<KEYTYPE,DATATYPE>::add(const KEYTYPE &k, const DATATYPE & x)
{add(k,x,root);}

template <class KEYTYPE,class DATATYPE>
void BinTree<KEYTYPE,DATATYPE>::add(const KEYTYPE &k)
{DATATYPE x;
 add(k,x);
}

template <class KEYTYPE,class DATATYPE>
void BinTree<KEYTYPE,DATATYPE>::iteratorReset()
{keyLoc=root;
 while(keyLoc!=NULL && keyLoc->child[LEFT]!=NULL)
   keyLoc=keyLoc->child[LEFT];
}

template <class KEYTYPE,class DATATYPE>
KEYTYPE BinTree<KEYTYPE,DATATYPE>::nextKey()
{Node *loc;
 KEYTYPE temp;
 check(!atEnd(),"(nextKey()) non-existent entry");
 temp=keyLoc->key;
 keyLoc=NULL;
 loc=root;
 while(loc!=NULL)
   if(loc->key<=temp)
     loc=loc->child[RIGHT];
   else
     {keyLoc=loc;
     loc=loc->child[LEFT];
     }
 return temp;
}
  
template <class KEYTYPE,class DATATYPE>
int BinTree<KEYTYPE,DATATYPE>::size()const
{return size(root);}

template <class KEYTYPE,class DATATYPE>
bool BinTree<KEYTYPE,DATATYPE>::atEnd()const
{return keyLoc==NULL;}

template <class KEYTYPE,class DATATYPE>
bool BinTree<KEYTYPE,DATATYPE>::has(const KEYTYPE &k)const
{return has(k,root);}

template <class KEYTYPE,class DATATYPE>
BinTree<KEYTYPE,DATATYPE> & BinTree<KEYTYPE,DATATYPE>::operator +=(const KEYTYPE &k)
{add(k);
 return *this;
}

template <class KEYTYPE,class DATATYPE>
DATATYPE & BinTree<KEYTYPE,DATATYPE>::operator[](const KEYTYPE& k)
{Node *temp;
 temp=root;
 while(temp!=NULL && temp->key!=k)
   if(k<temp->key)
     temp=temp->child[LEFT];
   else
     temp=temp->child[RIGHT];
 check(temp!=NULL, "([]) missing entry");
 return temp->data;
}

template <class KEYTYPE,class DATATYPE>
DATATYPE  BinTree<KEYTYPE,DATATYPE>::operator[](const KEYTYPE& k)const
{Node *temp;
 temp=root;
 while(temp!=NULL && temp->key!=k)
   if(k<temp->key)
     temp=temp->child[LEFT];
   else
     temp=temp->child[RIGHT];
 check(temp!=NULL, "([]) missing entry");
 return temp->data;
}
  
template <class KEYTYPE,class DATATYPE>
BinTree<KEYTYPE,DATATYPE>  BinTree<KEYTYPE,DATATYPE>::operator=(const BinTree&b)
{if(this!=&b)
  {destroy(root);
   copy(b.root,root);
  }
 return *this;
}


template <class KEYTYPE,class DATATYPE>
void BinTree<KEYTYPE,DATATYPE>::destroy(Node *&rt)
{if(rt!=NULL)
  {for(int j=0;j<2;j++)
    destroy(rt->child[j]);
  delete rt;
  rt=NULL;
  }
}

template <class KEYTYPE,class DATATYPE>
int BinTree<KEYTYPE,DATATYPE>::size(Node *rt)
{if(rt==NULL)
  return 0;
else
  return 1+size(rt->child[LEFT])+size(rt->child[RIGHT]);
} 

template <class KEYTYPE,class DATATYPE>
void BinTree<KEYTYPE,DATATYPE>::add(
	    const KEYTYPE &k,const DATATYPE& x, Node *&rt)
{if(rt==NULL)
  {rt=new Node(k,x);
  check(rt!=NULL,"(add()) heap overflow");
  }
else
  {check(rt->key!=k,"(add()) duplicate key");
  add(k,x,rt->child[int(k>rt->key)]);
  }
} 

template <class KEYTYPE,class DATATYPE>
void BinTree<KEYTYPE,DATATYPE>::copy(Node * oldRt,Node*&newRt)
{if(oldRt==NULL)
  newRt=NULL;
else
  {newRt=new Node(oldRt->key,oldRt->data);
  check(newRt!=NULL,"(copy()) heap overflow");
  for(int j=0;j<2; j++)
    copy(oldRt->child[j],newRt->child[j]);
  }
}

template <class KEYTYPE,class DATATYPE>
bool BinTree<KEYTYPE,DATATYPE>::has(const KEYTYPE &k,Node *rt)const
{return rt!=NULL
   && (rt->key==k ||
       has(k,rt->child[rt->key<k]));
}

template <class KEYTYPE,class DATATYPE>
double & BinTree<KEYTYPE,DATATYPE>::getEntry(const KEYTYPE &k,Node * rt)
{check(rt!=NULL,"(getEntry()) missing key");
 if(rt->key==k)
   return rt->data;
 else
   return getEntry(k,rt->child[int(k>rt->key)]);
}

template <class KEYTYPE,class DATATYPE>
void BinTree<KEYTYPE,DATATYPE>::check(bool b,char *mess)
{if(!b)
  {TRACE1("ERROR: %s",mess);
  exit(1);
  }
} 


template <class KEYTYPE,class DATATYPE>
void BinTree<KEYTYPE,DATATYPE>::remove(const KEYTYPE &k)
{keyLoc=NULL;
  remove(k,root);}

template <class KEYTYPE,class DATATYPE>
  int BinTree<KEYTYPE,DATATYPE>::countLarger(const KEYTYPE &k)const
{return countLarger(k,root);}

template <class KEYTYPE,class DATATYPE>
  int BinTree<KEYTYPE,DATATYPE>::countSmaller(const KEYTYPE & k)const
{return countSmaller(k,root);}

template <class KEYTYPE,class DATATYPE>
void BinTree<KEYTYPE,DATATYPE>::remove(const KEYTYPE & k,Node*&rt)
{Node *temp,*tempa;
 check(rt!=NULL,"(remove() nonexistent key");
 if(k<rt->key)
   remove(k,rt->child[LEFT]);
 else if(k>rt->key)
   remove(k,rt->child[RIGHT]);
 else
   {temp=rt;
   if(rt->child[LEFT]==NULL)
     {rt=rt->child[RIGHT];
     delete temp;
     }
   else if(rt->child[LEFT]->child[RIGHT]==NULL)
     {rt->child[LEFT]->child[RIGHT]=rt->child[RIGHT];
     rt=rt->child[LEFT];
     delete temp;
     }
   else
     {temp=temp->child[LEFT];
     while(temp->child[RIGHT]->child[RIGHT]!=NULL)
       temp=temp->child[RIGHT];
     rt->key=temp->child[RIGHT]->key;
     rt->data=temp->child[RIGHT]->data;
     tempa=temp->child[RIGHT];
     temp->child[RIGHT]=temp->child[RIGHT]->child[LEFT];
     delete tempa;
     }
   }
}


template <class KEYTYPE,class DATATYPE>
  int BinTree<KEYTYPE,DATATYPE>::countLarger(const KEYTYPE &k,Node *rt)const
{if(rt==NULL)
  return 0;
else if(k>=rt->key)
    return countLarger(k,rt->child[RIGHT]);
 else return
	1+countLarger(k,rt->child[LEFT])+countLarger(k,rt->child[RIGHT]);
}  
      
template <class KEYTYPE,class DATATYPE>
int BinTree<KEYTYPE,DATATYPE>::countSmaller(const KEYTYPE &k,Node *rt)const
{if(rt==NULL)
  return 0;
else if(k<=rt->key)
    return countSmaller(k,rt->child[LEFT]);
 else return
	1+countSmaller(k,rt->child[LEFT])+countSmaller(k,rt->child[RIGHT]);
}  
  
template <class KEYTYPE,class DATATYPE>
bool BinTree<KEYTYPE,DATATYPE>::sameTree(const BinTree &b)
{return sameTree(root,b.root);}

template <class KEYTYPE,class DATATYPE>
bool BinTree<KEYTYPE,DATATYPE>::sameTree(Node *a, Node*b)
{return a==NULL && b==NULL ||
   a!=NULL && b!=NULL && a->key==b->key && a->data==b->data &&
   sameTree(a->child[LEFT],b->child[LEFT])
   && sameTree(a->child[RIGHT],b->child[RIGHT]);
} 

#endif







