/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
#ifndef _MODS_COMMON_H_
#define _MODS_COMMON_H_

#define MIDDLEWARE_OF_DIRECTORY_SERVICE

//#define MODS_SIZEOF(Oid) (sizeof(Oid)/sizeof(Oid[0]))

typedef int ModsInteger; 
typedef bool ModsBool;
typedef char * ModsString;
typedef unsigned long ModsId;
typedef unsigned long ModsSequence;
typedef unsigned short ModsType;
typedef unsigned short ModsLevel;
typedef unsigned short ModsHelp;
typedef time_t	ModsTime;

typedef struct {
	UINT px_;
	UINT py_;
	UINT pz_;
} ModsPosition;

typedef struct {
	ModsId		 id;
	ModsPosition pos;
	ModsSequence seq;
	ModsType	 ntype;
	ModsLevel	 nlevel;
	ModsHelp	 nhelp;
} ModsFire;

typedef struct {
	ModsId		 id;
	ModsPosition pos;
	ModsSequence seq;
	ModsType	 ntype;
	ModsBool	 bCameraReady;
} ModsFFighter;

typedef struct {
	ModsFFighter ffighter;
	ModsTime	 texpire;
} ModsSFighter;

typedef struct {
	ModsId		id;
//	ModsType	seq;
} ModsVideo;

typedef struct {
	ModsId		id;
} ModsCBox;

typedef struct {
	UINT idLength;
	UINT * ids; 
} MODSIdentifier;

// template MIB entry
typedef struct MIB_ENTRY {
	MODSIdentifier	asnOid;
	void *			pStorageValue;
	CHAR *			szStorageName;
	BYTE			chType;
	UINT			unAccess;
	UINT			unProperty;
	MIB_ENTRY*		pMibNext;
	MIB_ENTRY*		pMibSon;
} MIB_ENTRY;

VOID MIBEntryInit(MIB_ENTRY*);
VOID MIBEntryRelease(MIB_ENTRY*);
VOID MIBTableRelease(MIB_ENTRY*);

// find and return the entry in current table with the identifier
// return NULL if not found
MIB_ENTRY* MODSFindEntry(MIB_ENTRY*, MODSIdentifier);
MIB_ENTRY* MODSFindParent(MIB_ENTRY*, MODSIdentifier);
MIB_ENTRY* MODSFindPrev(MIB_ENTRY*, MODSIdentifier);

// return zero if tbId == fid
// return positive if fid is in tbid
// return negative if fid is not in tbId
int MIBIDCompare(MODSIdentifier tbId, MODSIdentifier fId);

// Add a new entry to table
VOID MODSAddEntry(MIB_ENTRY*, MIB_ENTRY*);
// Edit a existing entry in the table
VOID MODSEditEntry(MIB_ENTRY*, MIB_ENTRY*);
// Delete an entry
VOID MODSDelEntry(MIB_ENTRY*, MIB_ENTRY*);
// Set the buffer based on pent information
VOID MODSSetBuffer(MIB_ENTRY* pent, char * buf, UINT * slen);
// Set all the trap information based on the pent table
VOID MODSSetTrap(MIB_ENTRY* pent, char * buf, UINT * slen, UINT * pntrap);

// set device oid {0, 1}
UINT * MODSSetDeviceOid();
UINT * MODSSetVideoOid();
UINT * MODSSetChatBoxOid();
UINT * MODSSetDisplayOid();
UINT * MODSSetFireOid();
UINT * MODSSetFFighterOid();
UINT * MODSSetMessageOid();
UINT * MODSSetPathOid();

MODSIdentifier MODSSetFireEntry();
BOOL IsMODSFireEntry(MODSIdentifier id);
MODSIdentifier MODSSetFFighterEntry();
BOOL IsMODSFFighterEntry(MODSIdentifier id);
MODSIdentifier MODSSetVideoEntry();
BOOL IsMODSVideoEntry(MODSIdentifier id);
MODSIdentifier MODSSetChatBoxEntry();
BOOL IsMODSChatBoxEntry(MODSIdentifier id);
MODSIdentifier MODSSetPathEntry();
BOOL IsMODSPathEntry(MODSIdentifier id);
BOOL MODSCheckIdExist(MIB_ENTRY*, MIB_ENTRY*);
BOOL MODSCheckIdentExist(MIB_ENTRY*, MIB_ENTRY*);

#endif //_MODS_COMMON_H_