/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"
#include "AwareActuator.h"

CAwareActuator::CAwareActuator(void)
{
	act_input	= NULL;
	input_len	= 0;
	act_output	= NULL;
	output_len	= 0;
	m_pConfCompnArray.RemoveAll();

	// active
	m_bSyncRequired	= false;
	m_syncIndex		= 0;

	// reactive
	outString		= NULL;
	m_remoteAddr	= 0;	
}

CAwareActuator::~CAwareActuator(void)
{
	if (outString != NULL) free(outString);
	for (INT_PTR i = 0; i< m_pConfCompnArray.GetSize(); i++) 
		delete m_pConfCompnArray[i];
	m_pConfCompnArray.RemoveAll();
}

void CAwareActuator::AddCompt(CRecfnCompntPtr p)
{
	m_pConfCompnArray.Add(p);
}

void CAwareActuator::SetInput(void * data, size_t len)
{
	act_input = data;
	input_len = len;
}

void CAwareActuator::GetOutput(void ** data, size_t * len)
{
	if (data != NULL) *data = act_output;
	if (len != NULL) *len  = output_len;
}

void CAwareActuator::Active()
{
	for (INT_PTR i=0; i<m_pConfCompnArray.GetSize(); i++)
		for (INT_PTR j=0; j<m_pConfCompnArray[i]->m_pParamArray.GetSize(); j++)
			m_pConfCompnArray[i]->GetComponent()->SetConfParam(m_pConfCompnArray[i]->m_pParamArray[j]);
}

void CAwareActuator::Start()
{
	void * data_out;
	size_t len_out;

	m_pConfCompnArray[0]->GetComponent()->Input(act_input, input_len);
	m_pConfCompnArray[0]->GetComponent()->Start();
	m_pConfCompnArray[0]->GetComponent()->Output(&data_out, &len_out);
	for (INT_PTR i=1; i<m_pConfCompnArray.GetSize(); i++) {
		m_pConfCompnArray[i]->GetComponent()->Input(data_out, len_out);	
		m_pConfCompnArray[i]->GetComponent()->Start();
		m_pConfCompnArray[i]->GetComponent()->Output(&data_out, &len_out);
	}

	act_output = data_out;
	output_len = len_out;
}

void CAwareActuator::Process(void * in_data, size_t in_len, void ** out_data, size_t *out_len)
{
	void * data_out;
	size_t len_out;

	m_pConfCompnArray[0]->GetComponent()->Input(in_data, in_len);
	m_pConfCompnArray[0]->GetComponent()->Start();
	m_pConfCompnArray[0]->GetComponent()->Output(&data_out, &len_out);

	for (INT_PTR i=1; i<m_pConfCompnArray.GetSize(); i++) {
		m_pConfCompnArray[i]->GetComponent()->Input(data_out, len_out);	
		m_pConfCompnArray[i]->GetComponent()->Start();
		m_pConfCompnArray[i]->GetComponent()->Output(&data_out, &len_out);
	}

	act_output = data_out;
	output_len = len_out;
	if (out_data != NULL) {
		*out_data = data_out;
		*out_len  = len_out;
	}
}

void CAwareActuator::Stop()
{
//	StopThread();
}

void CAwareActuator::RequireSync(bool b)
{
	m_bSyncRequired = b;
}

bool CAwareActuator::CheckSyncReq()
{
	return m_bSyncRequired;
}

char * CAwareActuator::ToString()
{
	if (outString != NULL) return outString;
	
	size_t len = 0, max_len = m_pConfCompnArray.GetSize() * MAX_COMPONENT_SIZE;
	outString = (char *)malloc(max_len);

	len += sizeof(len);
	size_t cmplen = m_pConfCompnArray.GetSize();
	memcpy_s(outString+len, max_len-len, &cmplen, sizeof(cmplen));
	len += sizeof(cmplen);
	for (size_t i=0; i<cmplen; i++) {
		strcpy_s(outString+len, max_len - len, m_pConfCompnArray[i]->GetName().get());
		len += strlen(m_pConfCompnArray[i]->GetName().get()) + 1;
		size_t paramlen = m_pConfCompnArray[i]->m_pParamArray.GetSize();
		memcpy_s(outString+len, max_len-len, &paramlen, sizeof(paramlen));
		len += sizeof(paramlen);
		for (size_t j=0; j<paramlen; j++) {
			strcat_s(outString+len, max_len-len, m_pConfCompnArray[i]->m_pParamArray[j].key.get());
			len += strlen(m_pConfCompnArray[i]->m_pParamArray[j].key.get()) + 1;
			strcat_s(outString+len, max_len-len, m_pConfCompnArray[i]->m_pParamArray[j].value.get());
			len += strlen(m_pConfCompnArray[i]->m_pParamArray[j].value.get()) + 1;
		}
	}
	memcpy_s(outString, max_len, &len, sizeof(len));
	return outString;
}

void CAwareActuator::FromString(char * inStr)
{
	for (INT_PTR i=0; i<m_pConfCompnArray.GetSize(); i++)
		delete m_pConfCompnArray[i];
	m_pConfCompnArray.RemoveAll();

	char name[MAX_STRING_LEN], key[MAX_STRING_LEN], value[MAX_STRING_LEN];

	size_t len = 0, max_len = *(size_t *)inStr;
	len += sizeof(len);
	size_t cmplen = *(size_t *)(inStr + len);
	len += sizeof(cmplen);
	for (size_t i=0; i<cmplen; i++) {
		CRecfnCompntPtr pCompnt = new CReconfigComponent();
		strcpy_s(name, MAX_STRING_LEN, inStr + len);
		len += strlen(name) + 1;
		pCompnt->SetName(name);
		size_t paramlen = *(size_t *)(inStr + len);
		len += sizeof(paramlen);
		for (size_t j=0; j<paramlen; j++) {
			strcpy_s(key, MAX_STRING_LEN, inStr + len);
			len += strlen(key) + 1;
			strcpy_s(value, MAX_STRING_LEN, inStr + len);
			len += strlen(value) + 1;
			pCompnt->AddParam(key, value);
		}
		m_pConfCompnArray.Add(pCompnt);
	}
	if (len != max_len) {
		AfxMessageBox(_T("Synchronization error!"));
		exit(0);
	}
}

void CAwareActuator::SetSyncIndex(BYTE m)
{
	m_syncIndex = m;
}

BYTE CAwareActuator::GetSyncIndex()
{
	return m_syncIndex;
}

void CAwareActuator::SetRemoteAddr(unsigned long addr)
{
	m_remoteAddr = addr;
}

unsigned long CAwareActuator::GetRemoteAddr()
{
	return m_remoteAddr;
}

void CAwareActuator::format(char *buf)
{
	strcat_s(buf, 512, m_pConfCompnArray[0]->GetName().get());
	for (INT_PTR i=1; i<m_pConfCompnArray.GetSize(); i++) {
		strcat_s(buf, 512, " --> ");
		strcat_s(buf, 512, m_pConfCompnArray[i]->GetName().get());
	}
}