/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"
#include "AwareMetaObject.h"

CCriticalSection CAwareMetaObject::m_dataLock;
CCriticalSection CAwareMetaObject::m_syncLock;
int gnActionDoneEvent = RegisterWindowMessageA(szActionDoneEvent);
int gnReceivDoneEvent = RegisterWindowMessageA(szReceivDoneEvent);
int gnArchChangeEvent = RegisterWindowMessageA(szArchChangeEvent);

/*****************************************************************************************************/
// Constructor and Deconstructor
CAwareMetaObject::CAwareMetaObject(void)
{
	act_input	= NULL;
	input_len	= 0;
	act_output	= NULL;
	output_len	= 0;
	act_hWnd	= NULL;

	proc_output = NULL;
	proc_outlen = 0;

	m_actvIndex	= 0;
	m_recvIndex	= 0;

	m_hSourceEvent	= NULL;
	m_threadWaiting	= NULL;
	m_threadReceiving = NULL;
	m_statusWaitingThread	= CLOSED;
	m_statusReceivingThread	= CLOSED;
	m_statusSync	= NONE;

	m_pEventArray.RemoveAll();
	m_pActArray.RemoveAll();
	m_pSyncActArray.RemoveAll();
}

CAwareMetaObject::~CAwareMetaObject(void)
{
	Deconstruct();
}

/*****************************************************************************************************/
// Setparameter and public functions
void CAwareMetaObject::SetHWnd(CWnd *pWnd)
{
	act_hWnd = pWnd->m_hWnd;
}

void CAwareMetaObject::SetCompntAppParam(CWord cName, CWord key, void * value)
{
	for (INT_PTR i=0; i<m_pActiveAwarelets.GetSize(); i++)
		if (cName == m_pActiveAwarelets[i]->GetName()) {
			m_pActiveAwarelets[i]->GetComponent()->SetAppParam(key, value);
			break;
		}
	for (INT_PTR i=0; i<m_pReactiveAwarelets.GetSize(); i++)
		if (cName == m_pReactiveAwarelets[i]->GetName()) {
			m_pReactiveAwarelets[i]->GetComponent()->SetAppParam(key, value);
			break;
		}
	for (INT_PTR i=0; i<m_pAwareTools.GetSize(); i++)
		if (cName == m_pAwareTools[i]->GetName()) {
			m_pAwareTools[i]->SetAppParam(key, value);
			break;
		}
}

void CAwareMetaObject::SetInput(void * data, size_t len)
{
	act_input = data;
	input_len = len;
}

void CAwareMetaObject::GetOutput(void ** data, size_t * len)
{
	if (data != NULL) *data = act_output;
	if (len != NULL) *len  = output_len;
}

void CAwareMetaObject::Process()
{
	if (m_pActArray.GetSize() <= m_actvIndex) return;

	CSingleLock csl(&CAwareMetaObject::m_dataLock);
	csl.Lock();	//ENTER CRITICAL REGION

	m_pActArray[m_actvIndex]->SetInput(act_input, input_len);
	m_pActArray[m_actvIndex]->Start();
	m_pActArray[m_actvIndex]->GetOutput(&act_output, &output_len);

	if(m_pActArray[m_actvIndex]->CheckSyncReq())  {
		if (m_statusSync < REQUESTED)  return;
		if (m_statusSync < CONNECTED) { RequestSync((sockaddr *)&m_remoteAddr); return; }
		char buf[MAX_BUFF_SIZE];
		*(LPBYTE)buf = m_pActArray[m_actvIndex]->GetSyncIndex();
		memcpy_s(buf+sizeof(BYTE), MAX_BUFF_SIZE-sizeof(BYTE), act_output, output_len);
		sendto(m_sock, buf, int(sizeof(BYTE)+output_len), 0, (struct sockaddr *)&m_remoteAddr,sizeof(m_remoteAddr));
	}

	csl.Unlock();	//EXIT CRITICAL REGION

	OnReceive();
}

void CAwareMetaObject::ProcessActv(void * in_data, size_t in_len, void ** out_data, size_t *out_len)
{
	if (m_pActArray.GetSize() <= m_actvIndex) return;

	CSingleLock csl(&CAwareMetaObject::m_dataLock);
	csl.Lock();	//ENTER CRITICAL REGION

	m_pActArray[m_actvIndex]->SetInput(in_data, in_len);
	m_pActArray[m_actvIndex]->Start();
	m_pActArray[m_actvIndex]->GetOutput(&act_output, &output_len);

	if (out_data != NULL) {
		*out_data	= act_output;
		*out_len	= output_len;
	}

	if(m_pActArray[m_actvIndex]->CheckSyncReq())  {
		if (m_statusSync < REQUESTED)  return;
		if (m_statusSync < CONNECTED) { RequestSync((sockaddr *)&m_remoteAddr); return; }
		char buf[MAX_BUFF_SIZE];
		*(LPBYTE)buf = m_pActArray[m_actvIndex]->GetSyncIndex();
		memcpy_s(buf+sizeof(BYTE), MAX_BUFF_SIZE-sizeof(BYTE), act_output, output_len);
		sendto(m_sock, buf, int(sizeof(BYTE)+output_len), 0, (struct sockaddr *)&m_remoteAddr,sizeof(m_remoteAddr));
	}

	csl.Unlock();	//EXIT CRITICAL REGION

	OnReceive();
}

void CAwareMetaObject::ProcessRecv(sockaddr * paddr, void * in_data, size_t in_len, void ** out_data, size_t *out_len)
{
	BYTE index = *(LPBYTE)in_data;
	if (m_pReActArray.GetSize() <= index || 
		m_pReActArray[index]->GetRemoteAddr() != ((sockaddr_in *)paddr)->sin_addr.s_addr) {
			BYTE type = SYNCHRONIZATION_ASK;
			sendto(m_sock, (char *)&type, sizeof(type), 0, paddr, sizeof(sockaddr_in));
			return;
	}

	if (m_recvIndex != index) {
		m_recvIndex = index;
		m_pReActArray[m_recvIndex]->Active();
		FormatOutput(1);
	}
	m_pReActArray[m_recvIndex]->Process((char *)in_data + sizeof(index), in_len - sizeof(index), &proc_output, &proc_outlen);
/*	m_pReActArray[m_recvIndex]->SetInput((char *)in_data + sizeof(index), in_len - sizeof(index));
	m_pReActArray[m_recvIndex]->Start();
	m_pReActArray[m_recvIndex]->GetOutput(&proc_output, &proc_outlen);
*/
	if (out_data != NULL) {
		*out_data	= proc_output;
		*out_len	= proc_outlen;
	}

	OnReceive(1);
}


void CAwareMetaObject::AddObject(CEventPtr pEvent, CActuatorPtr pAct, CActuatorPtr pSyncAct)
{
	SetActuatorCompnt(pAct, &m_pActiveAwarelets);
	m_pEventArray.Add(pEvent);
	if (pSyncAct != NULL) pAct->RequireSync(true);
	m_pActArray.Add(pAct);
	m_pSyncActArray.Add(pSyncAct);
}

void CAwareMetaObject::Construct()
{
	int size = (int)m_pEventArray.GetSize();
	if (size != m_pActArray.GetSize()) return;
	if (size > 0) m_hSourceEvent = new HANDLE[size];
	char lpName[64];
	for (int i=0; i<size; i++) {
		sprintf_s(lpName, 64, "MetaObjectThread%d\n", i);
		m_hSourceEvent[i] = CreateEventA(NULL, FALSE, FALSE, lpName); 
		if (m_hSourceEvent[i] == NULL) { 
			AfxMessageBox(_T("CreateEvent error! \n")); 
			ExitProcess(0); 
		}
		m_pEventArray[i]->SetListener(m_hSourceEvent[i]);
//		m_pEventArray[i]->Start();
	}
//	if (size > 0) StartWaitingThread();
	if (m_pReactiveAwarelets.GetSize() > 0) InitReadySync();
}

void CAwareMetaObject::Start()
{
	INT_PTR i, size = m_pActArray.GetSize();

	if (size > 0) StartWaitingThread();	// waiting for event notification
	for (i=0; i<size; i++)
		m_pEventArray[i]->Start();

//	if (m_pReactiveAwarelets.GetSize() > 0) InitReadySync();
}

void CAwareMetaObject::Stop()
{
	StopWaitingThread();
	for (INT_PTR i=0; i<m_pActArray.GetSize(); i++)
		m_pEventArray[i]->Stop();
}

/*****************************************************************************************************/
// Protected functions for dynamic reconfiguration
void CAwareMetaObject::StartWaitingThread()
{
	m_threadWaiting = CreateThread(NULL, 0, EventWaitingThread, this, 0, NULL);
	m_statusWaitingThread	= RUNNING;
	return;
}

DWORD WINAPI CAwareMetaObject::EventWaitingThread(LPVOID arg)
{
	CAwareMetaObject * thisClass = (CAwareMetaObject *)arg;
	thisClass->EventWaitingFun();
	return 0;
}

void CAwareMetaObject::EventWaitingFun()
{
	DWORD dwEvent, num = (DWORD)m_pEventArray.GetSize();
	while(m_statusWaitingThread == RUNNING) {
		dwEvent = WaitForMultipleObjects(num, m_hSourceEvent, FALSE, INFINITE);
		if (m_statusWaitingThread != RUNNING) break;

		CSingleLock csl(&CAwareMetaObject::m_dataLock);
		csl.Lock();	//ENTER CRITICAL REGION
		if (dwEvent-WAIT_OBJECT_0 != m_actvIndex) {
			m_actvIndex = dwEvent - WAIT_OBJECT_0;
			m_pActArray[m_actvIndex]->Active();
			FormatOutput(0);
		}
		csl.Unlock();	//EXIT CRITICAL REGION

	}
	m_statusWaitingThread	= CLOSING;

	// do something else;

	// Self terminate the thread
	m_statusWaitingThread	= CLOSED;
	m_threadWaiting = NULL;
}

void CAwareMetaObject::Deconstruct()
{
	StopWaitingThread();
	CloseEventHandles();
	CloseSyncPart();
	ReleaseArchs();
	ReleaseCompnts();
}

void CAwareMetaObject::StopWaitingThread()
{
	int watchDog = 0;
	while (m_threadWaiting)
	{
		m_statusWaitingThread = CLOSING;
		SetEvent(m_hSourceEvent[0]);
		SetThreadPriority(m_threadWaiting, THREAD_PRIORITY_HIGHEST);
		if((::WaitForSingleObject(m_threadWaiting, 1000) != WAIT_TIMEOUT) ||
			watchDog >= THREAD_TERMINATION_MAXWAIT)
		{
			CloseHandle(m_threadWaiting);
			m_statusWaitingThread	= CLOSED;
			m_threadWaiting	= NULL;
		}
		watchDog++;
	}
}

void CAwareMetaObject::CloseEventHandles()
{
	if (m_hSourceEvent != NULL) {
		for (int i=0; i<m_pEventArray.GetSize(); i++) {
			m_pEventArray[i]->Stop();
//			m_pActArray[i]->Stop();
			CloseHandle(m_hSourceEvent[i]);
		}
		delete[] m_hSourceEvent;
		m_hSourceEvent = NULL;
	}
}

void CAwareMetaObject::ReleaseArchs()
{
	for (INT_PTR i=0; i<m_pActArray.GetSize(); i++) {
		delete m_pEventArray[i];
		delete m_pActArray[i];
		if (m_pSyncActArray[i] != NULL) delete m_pSyncActArray[i];
	}
	m_pEventArray.RemoveAll();
	m_pActArray.RemoveAll();
	m_pSyncActArray.RemoveAll();

	for (INT_PTR i=0; i<m_pReActArray.GetSize(); i++)
		delete m_pReActArray[i];
	m_pReActArray.RemoveAll();
}

void CAwareMetaObject::ReleaseCompnts() 
{
	for (INT_PTR i=0; i<m_pAwareTools.GetSize(); i++) 
		delete m_pAwareTools[i];

	m_pAwareTools.RemoveAll();
	for (INT_PTR i=0; i<m_pActiveAwarelets.GetSize(); i++) {
		HINSTANCE hInst = m_pActiveAwarelets[i]->GetComponent()->GetDllInst();
		delete m_pActiveAwarelets[i]->GetComponent();
		FreeLibrary(hInst);
		delete m_pActiveAwarelets[i];
	}
	m_pActiveAwarelets.RemoveAll();

	for (INT_PTR i=0; i<m_pReactiveAwarelets.GetSize(); i++) {
		HINSTANCE hInst = m_pReactiveAwarelets[i]->GetComponent()->GetDllInst();
		delete m_pReactiveAwarelets[i]->GetComponent();
		FreeLibrary(hInst);
		delete m_pReactiveAwarelets[i];
	}
	m_pReactiveAwarelets.RemoveAll();
}

void CAwareMetaObject::OnReceive(BYTE msg)
{
	switch(msg) {
	case 0:	// active message is processed
		PostMessage(act_hWnd, gnActionDoneEvent, 0, 0); break;
	case 1:	// received message is processed
		PostMessage(act_hWnd, gnReceivDoneEvent, 0, 0); break;
	case 2: // architecture is changed
		PostMessage(act_hWnd, gnArchChangeEvent, 0, 0); break;
	default:
		break;
	}
}

/*****************************************************************************************************/
// protected function for synchronization
void CAwareMetaObject::InitReadySync()
{
	StartNetwork();
	CreateSocket();
	StartReceivingThread();
}

bool CAwareMetaObject::StartNetwork()
{
    WSADATA wsaData;
    WORD wVersionRequested = MAKEWORD(2,2);
    int nRet;

	// Init WinSock
    nRet = WSAStartup(wVersionRequested, &wsaData);
	if (nRet != 0) {
		AfxMessageBox(_T("\nError initializing WinSock\n"));
		return FALSE;
	}

	// Check version
	if (wsaData.wVersion != wVersionRequested) {
		AfxMessageBox(_T("\nWinSock version not supported\n"));
		WSACleanup();
		return FALSE;
	}

	m_statusSync = INITIALIZED;
	return TRUE;
}

bool CAwareMetaObject::CreateSocket()
{
	if (m_statusSync < INITIALIZED) StartNetwork();
	m_sock = socket(AF_INET, SOCK_DGRAM, 0);
	if (m_sock == INVALID_SOCKET) {
		AfxMessageBox(_T("Socket Error: Can not create Socket!"));
		CloseNetwork();
		return FALSE;
	}
	m_statusSync = CREATED;


	//bind local address to socket
	memset(&m_localAddr,0,sizeof(m_localAddr));
	m_localAddr.sin_family = AF_INET;
	m_localAddr.sin_addr.s_addr = htonl(INADDR_ANY);
	m_localAddr.sin_port = htons(SOCKET_VIDEO_RECV_PORT);
	int nb = bind(m_sock,(struct sockaddr *)&m_localAddr,sizeof(m_localAddr));
	if (nb == SOCKET_ERROR) {
		AfxMessageBox(_T("Bind Error: Can not bind local address to Socket!"));
		CloseNetwork();
		return FALSE;
	}

	// set remote host information. Could be broadcast
	return TRUE;
}

void CAwareMetaObject::StartReceivingThread()
{
	m_threadReceiving = CreateThread(NULL, 0, SyncReceivingThread, this, 0, NULL);
	m_statusReceivingThread	= RUNNING;
}

DWORD WINAPI CAwareMetaObject::SyncReceivingThread(LPVOID arg)
{
	CAwareMetaObject * thisClass = (CAwareMetaObject *)arg;
	thisClass->SyncReceivingFun();
	return 0;
}

void CAwareMetaObject::SyncReceivingFun()
{
	char buff[MAX_BUFF_SIZE];

	struct sockaddr recvaddr;
	int rcvAddrLen = sizeof(recvaddr);
	int rcvLen;

	while(m_statusReceivingThread == RUNNING) {
		rcvLen = recvfrom(m_sock, buff, MAX_BUFF_SIZE, 0, (sockaddr *)&recvaddr, &rcvAddrLen);
		if (m_statusReceivingThread != RUNNING) break;
		if (rcvLen ==SOCKET_ERROR || rcvLen <=0) break;
		
		CSingleLock csl(&CAwareMetaObject::m_syncLock);
		csl.Lock();	//ENTER CRITICAL REGION
	
		ProcBuf(buff, rcvLen, &recvaddr);

		csl.Unlock();
	}

	m_statusReceivingThread = CLOSING;

	if(m_statusSync > DISCONNECTED)	CloseNetwork();

	// Self terminate the thread
	m_statusReceivingThread = CLOSED;
	m_threadReceiving = NULL;
}

void CAwareMetaObject::StopReceivingThread()
{
	int watchDog = 0;
	while (m_threadReceiving)
	{
		m_statusReceivingThread = CLOSING;
		SetThreadPriority(m_threadReceiving, THREAD_PRIORITY_HIGHEST);
		if((::WaitForSingleObject(m_threadReceiving, 1000) != WAIT_TIMEOUT) ||
			watchDog >= THREAD_END_MAXWAIT)
		{
			CloseHandle(m_threadReceiving);
			m_threadReceiving = NULL;
			m_statusReceivingThread = CLOSED;
		}
		watchDog++;
	}
}

void CAwareMetaObject::CloseSyncPart()
{
	if (m_statusSync == CONNECTED) {
		BYTE type = SYNCHRONIZATION_EXIT;
		sendto(m_sock, (char *)&type, sizeof(type), 0, (struct sockaddr *)&m_remoteAddr, sizeof(m_remoteAddr));
	}
	CloseNetwork();
	StopReceivingThread();
}


void CAwareMetaObject::CloseSocket()
{
	if (m_statusSync >= CREATED) {
		shutdown(m_sock, SD_BOTH);
		closesocket(m_sock);
		m_statusSync = INITIALIZED;
	}
}

void CAwareMetaObject::CloseNetwork()
{
	if (m_statusSync > INITIALIZED) CloseSocket();
	if (m_statusSync > DISCONNECTED) {
		WSACleanup();
		m_statusSync = DISCONNECTED;
	}
}

/*****************************************************************************************************/
// protected function for synchronization construction

void CAwareMetaObject::SyncTo(char * addr)
{
/*	LPHOSTENT remoteHost;
	if (isalpha(addr[0]))   // host address is a name
		remoteHost = gethostbyname(addr);
	else {
		dest = inet_addr(addr);
		remoteHost = gethostbyaddr((char *)&dest, 4, AF_INET);
	}
	m_remoteAddr.sin_family = AF_INET;
	memcpy((char *)&m_remoteaddr.sin_addr.s_addr, remoteHost->h_addr_list[0],remoteHost->h_length);
	m_remoteaddr.sin_port = htons(SOCKET_VIDEO_RECV_PORT);
*/
	memset(&m_remoteAddr,0,sizeof(m_remoteAddr));
	m_remoteAddr.sin_family = AF_INET;
	m_remoteAddr.sin_addr.s_addr = inet_addr(addr);
	m_remoteAddr.sin_port = htons(SOCKET_VIDEO_RECV_PORT);
	RequestSync((sockaddr *)&m_remoteAddr);
}

void CAwareMetaObject::RequestSync(sockaddr * paddr) {
	if (m_statusSync < CREATED) InitReadySync();
	if (m_statusSync < CONNECTED) {
		char * buf = (char *)malloc(MAX_BUFF_SIZE);
		size_t len = 0;

		ToSyncReqestInfo(buf, &len);

		sendto(m_sock, buf, int(len), 0, paddr, sizeof(sockaddr_in));
		if (m_statusSync < REQUESTED) m_statusSync = REQUESTED;
		free(buf);
	}
}

void CAwareMetaObject::ToSyncReqestInfo(char * buf, size_t * total_len)
{
	size_t len = 0; // *total_len;
	BYTE type = SYNCHRONIZATION_REQUEST, arraynum = (BYTE)m_pActArray.GetSize(), msgnum=0;

	memcpy_s(buf+len, MAX_BUFF_SIZE-len, &type, sizeof(type));	// set pack type
	len += sizeof(type);
	memcpy_s(buf+len, MAX_BUFF_SIZE-len, &arraynum, sizeof(arraynum));	// reserve pos for message number
	len += sizeof(arraynum);

	for (BYTE i=0; i<arraynum; i++) {
		if (m_pSyncActArray[i] == NULL) continue;
		msgnum ++;
		memcpy_s(buf+len, MAX_BUFF_SIZE-len, &i, sizeof(i));	// set request index of this arch
		len += sizeof(i);
		char *str = m_pSyncActArray[i]->ToString();	// get acutator synchinfo size
		size_t actlen = *(size_t *)str;
		memcpy_s(buf+len, MAX_BUFF_SIZE-len, str, actlen);	// set the actuator synchinfo
		len += actlen;
	}
	// reset message number and total buffer length
	memcpy_s(buf+(*total_len)+sizeof(type), MAX_BUFF_SIZE-(*total_len)-sizeof(type), &msgnum, sizeof(msgnum));
	*total_len = len;
}

void CAwareMetaObject::FromSyncReqestInfo(sockaddr * paddr, char* buf, size_t total_len)
{
	RemoveReactiveArchs((sockaddr_in *)paddr);
	char out[MAX_BUFF_SIZE]; 

	size_t len = 0, olen = 0; // *out_len;
	BYTE type, msgnum, id_req, id_resp;
	type = *(LPBYTE)(buf+len);		// get packet type
	len += sizeof(type);
	if (type != SYNCHRONIZATION_REQUEST) return;	// check package type

	type = SYNCHRONIZATION_RESPONSE;	// set response pack type
	memcpy_s(out+olen, MAX_BUFF_SIZE-olen, &type, sizeof(type));
	olen += sizeof(type);

	msgnum = *(LPBYTE)(buf+len);	// get message number
	len += sizeof(msgnum);

	memcpy_s(out+olen, MAX_BUFF_SIZE-olen, &msgnum, sizeof(msgnum));	// set response pack message num
	olen += sizeof(msgnum);

	for (BYTE i=0; i<msgnum; i++) {
		id_req = *(LPBYTE)(buf+len);	// get request index
		len += sizeof(id_req);

		memcpy_s(out+olen, MAX_BUFF_SIZE-olen, &id_req, sizeof(id_req));	// set response index
		olen += sizeof(id_req);
		id_resp = (BYTE)m_pReActArray.GetSize();	// get response index
		memcpy_s(out+olen, MAX_BUFF_SIZE-olen, &id_resp, sizeof(id_resp));	// set response index
		olen += sizeof(id_resp);

		CActuatorPtr pAct = new CAwareActuator();	// create a actuator
		pAct->FromString(buf+len);
		len += *(size_t*)(buf+len);
		pAct->SetRemoteAddr(((sockaddr_in *)paddr)->sin_addr.s_addr);
		SetActuatorCompnt(pAct, &m_pReactiveAwarelets);

		m_pReActArray.Add(pAct);
	}
	if (len != total_len) {
		AfxMessageBox(_T("Sync Request Info Error"));
		exit(0);
	}
	sendto(m_sock, out, int(olen), 0, paddr,sizeof(sockaddr_in));
}

void CAwareMetaObject::ToSyncResponseInfo(char * buf, size_t * total_len)
{
	
}

void CAwareMetaObject::FromSyncResponseInfo(sockaddr * paddr, char * buf, size_t total_len)
{
	size_t len = 0;
	BYTE type, msgnum, id_req, id_resp;
	type = *(LPBYTE)(buf+len);		// get packet type
	len += sizeof(type);
	if (type != SYNCHRONIZATION_RESPONSE) return;	// check package type

	msgnum = *(LPBYTE)(buf+len);	// get message number
	len += sizeof(msgnum);

	for (BYTE i=0; i<msgnum; i++) {
		id_req = *(LPBYTE)(buf+len);	// get request index
		len += sizeof(id_req);

		id_resp = *(LPBYTE)(buf+len);	// get request index
		len += sizeof(id_resp);

		m_pActArray[id_req]->SetSyncIndex(id_resp);
	}
	if (len != total_len) {
		AfxMessageBox(_T("Sync Request Info Error"));
		exit(0);
	}
	m_statusSync = CONNECTED;
}

void CAwareMetaObject::ProcBuf(char * buff, int rcvLen, sockaddr * paddr)
{
	BYTE type = *(LPBYTE)buff;

	switch(type){
	case SYNCHRONIZATION_REQUEST:
		FromSyncReqestInfo(paddr, buff, rcvLen); 
		break;
	case SYNCHRONIZATION_RESPONSE:
		FromSyncResponseInfo(paddr, buff, rcvLen); 
		break;
	case SYNCHRONIZATION_EXIT:
		RemoveReactiveArchs((sockaddr_in *)paddr);
		break;
	case SYNCHRONIZATION_ASK:
		m_statusSync = CREATED;
		RequestSync(paddr); 
		break;
	default:
		ProcessRecv(paddr, buff, rcvLen);
		break;
	}
}

void CAwareMetaObject::SetActuatorCompnt(CActuatorPtr pAct, CRecfnCompntPtrArray * pArray)
{
	INT_PTR i, j, n, m;
	for (i=0; i<pAct->m_pConfCompnArray.GetSize(); i++)
		for (j=0; j<pArray->GetSize(); j++)
			if (pAct->m_pConfCompnArray[i]->GetName() == pArray->GetAt(j)->GetName()) {
				pAct->m_pConfCompnArray[i]->SetComponent(pArray->GetAt(j)->GetComponent());
				for (n=0; n<pArray->GetAt(j)->m_pParamArray.GetSize(); n++) {
					for (m=0; m<pAct->m_pConfCompnArray[i]->m_pParamArray.GetSize(); m++)
						if (pArray->GetAt(j)->m_pParamArray[n].key == pAct->m_pConfCompnArray[i]->m_pParamArray[m].key)
							break;
					if (m == pAct->m_pConfCompnArray[i]->m_pParamArray.GetSize()) 
						pAct->m_pConfCompnArray[i]->m_pParamArray.Add(pArray->GetAt(j)->m_pParamArray[n]);
				}
			}
}

void CAwareMetaObject::RemoveReactiveArchs(sockaddr_in * paddr)
{
	for (int i=0; i<m_pReActArray.GetSize(); i++)
		if (m_pReActArray[i]->GetRemoteAddr() == paddr->sin_addr.s_addr) {
			CActuatorPtr pAct = m_pReActArray[i];
			m_pReActArray.RemoveAt(i);
			delete pAct;
			i--;
		}
}

void CAwareMetaObject::FormatOutput(int type)
{
	if (type == 0) {
		strcpy_s(fm_out, 1024, "Active Archtecture change: ");
		m_pActArray[m_actvIndex]->format(fm_out);
	}
	else if (type == 1) {
		strcpy_s(fm_out, 1024, "Reactive Archtecture change: ");
		m_pReActArray[m_recvIndex]->format(fm_out);
	}
	OnReceive(2);
}

void CAwareMetaObject::GetFormatV(char *buf)
{
	strcpy_s(buf, 1024, fm_out);
}
