/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"

#include <stdlib.h>

#include "AwareXMLParser.h"

/*****************************************************************************************************/
// Constructor and Deconstructor

CAwareXMLParser::CAwareXMLParser()
{
	m_cEventTag = '0';
	m_pCMClient = NULL;
}

CAwareXMLParser::CAwareXMLParser(CWord w, CMetaObjectPtr pObject) : m_xmlFile(w), m_pMetaObject(pObject)
{
	CAwareXMLParser();
}

CAwareXMLParser::CAwareXMLParser(CMetaObjectPtr pObject, CWord w) : m_pMetaObject(pObject), m_xmlFile(w)
{
	CAwareXMLParser();
}


CAwareXMLParser::~CAwareXMLParser()
{
	if (m_pCMClient != NULL) delete m_pCMClient;
}

/*****************************************************************************************************/
// Public:

void CAwareXMLParser::SetXmlFile(CWord w)
{
	m_xmlFile = w;
}

void CAwareXMLParser::SetMetaObject(CMetaObjectPtr pObject)
{
	m_pMetaObject = pObject;
}

bool CAwareXMLParser::Parse()
{
	InitializeXML(m_xmlFile.get());
	StartXMLParse(m_pDocRoot);
	return true;
}

/*****************************************************************************************************/
// Protected 1:

bool CAwareXMLParser::InitializeXML(char * xmlfile)
{
	CoInitialize(NULL);
	try {
		MSXML2::IXMLDOMDocumentPtr m_pDOMDoc = NULL;
		
		HRESULT hr = m_pDOMDoc.CreateInstance(__uuidof(MSXML2::DOMDocument40));
		m_pDOMDoc->async = VARIANT_FALSE;
		
		//hr = objDOMDoc->load("addressbook.xml");
		_variant_t varXml(xmlfile);
		_variant_t varOut((bool)TRUE);

		varOut = m_pDOMDoc->load(varXml);

		if ((BOOL)varOut == FALSE)	{
			MSXML2::IXMLDOMParseErrorPtr  pError;
			pError = m_pDOMDoc->parseError;
			_bstr_t parseError =_bstr_t("At line ")+ _bstr_t(pError->Getline()) + _bstr_t("\n")+ _bstr_t(pError->Getreason());
			MessageBox(NULL, parseError, _T("Parse Error"), MB_OK);
			return false;
		} else {
			m_pDocRoot = m_pDOMDoc->documentElement;
		}
	}
	catch(_com_error &e) 
	{		
		dump_com_error(e);
		return false;
	}
	return true;
}

void CAwareXMLParser::StartXMLParse(IXMLDOMNodePtr pNode)
{
	IXMLDOMNodePtr pChild, pNext;
	BSTR nodeName;
	char chName[MAX_STRING_SIZE];

	pNode->get_firstChild(&pChild);
	while (pChild != NULL) {
		pChild->get_nodeName(&nodeName);

		wcstombs_s(NULL, chName, MAX_STRING_SIZE, nodeName, MAX_STRING_SIZE);
		if (strcmp(chName, "DirectoryServer") == 0){
			ParseDirectoryServer(pChild);
		}
		else if (strcmp(chName, "ComponentManager") == 0) {
			ParseComponentManager(pChild);
		}
		else if (strcmp(chName, "AwareTools") == 0) {
			ParseAwareTools(pChild);
		}
		else if (strcmp(chName, "Marchlets") == 0) {
			ParseAwarelets(pChild);
		}
		else if (strcmp(chName, "Rules") == 0) {
			ParseRules(pChild);
		}
		else check(false, _T("XML Category Error"));

		pChild->get_nextSibling(&pNext);
		pChild = pNext;
	}
}

void CAwareXMLParser::ParseDirectoryServer(IXMLDOMNodePtr pDS)
{
}

void CAwareXMLParser::ParseComponentManager(IXMLDOMNodePtr pCM)
{
	IXMLDOMNodePtr pChild, pNext;
	BSTR nodeName, nodeValue;
	char chName[MAX_STRING_SIZE], chIP[MAX_STRING_SIZE], chPort[MAX_STRING_SIZE];

	m_pCMClient = new CCMClient();

	pCM->get_firstChild(&pChild);
	while (pChild != NULL) {
		pChild->get_nodeName(&nodeName);
		wcstombs_s(NULL, chName, MAX_STRING_SIZE, nodeName, MAX_STRING_SIZE);
		if (strcmp(chName, "ip") == 0){
			pChild->get_text(&nodeValue);
			wcstombs_s(NULL, chIP, MAX_STRING_SIZE, nodeValue, MAX_STRING_SIZE);
		}
		else if (strcmp(chName, "port") == 0) {
			pChild->get_text(&nodeValue);
			wcstombs_s(NULL, chPort, MAX_STRING_SIZE, nodeValue, MAX_STRING_SIZE);
		}
		else check(false, _T("XML Category Error"));

		pChild->get_nextSibling(&pNext);
		pChild = pNext;
	}

	m_pCMClient->SetRemoteAddr(chIP, atoi(chPort));
}

void CAwareXMLParser::ParseAwareTools(IXMLDOMNodePtr pAwares)
{
	IXMLDOMNodePtr pChild, pNext;
	pAwares->get_firstChild(&pChild);
	while (pChild != NULL) {
		CAwareSrcPtr p = GetSingleAwareTool(pChild);
		m_pMetaObject->m_pAwareTools.Add(p);

		pChild->get_nextSibling(&pNext);
		pChild = pNext;
	}
}

void CAwareXMLParser::ParseAwarelets(IXMLDOMNodePtr pComps)
{
	IXMLDOMNodePtr pChild, pNext;
	pComps->get_firstChild(&pChild);
	while (pChild != NULL) {
		AWARELETTYPE type;
		CRecfnCompntPtr p = GetSingleAwarelet(pChild, type);
		switch(type){
		case ACTIVE:
			m_pMetaObject->m_pActiveAwarelets.Add(p);
			break;
		case REACTIVE:
			m_pMetaObject->m_pReactiveAwarelets.Add(p);
			break;
		case BOTH:
			break;
		}

		pChild->get_nextSibling(&pNext);
		pChild = pNext;
	}
}

void CAwareXMLParser::ParseRules(IXMLDOMNodePtr pRules)
{
	IXMLDOMNodePtr pChild, pNext;
	
	pRules->get_firstChild(&pChild);
	while (pChild != NULL) {
		ParseSingleRule(pChild);

		pChild->get_nextSibling(&pNext);
		pChild = pNext;
	}
}

void CAwareXMLParser::ParseSingleRule(IXMLDOMNodePtr pRule)
{
	CEventPtr pEvent		= NULL;
	CActuatorPtr pAct		= NULL;
	CActuatorPtr pSyncAct	= NULL;

	IXMLDOMNodePtr pChild, pNext;
	BSTR nodeName;
	char chName[MAX_STRING_SIZE];


	pRule->get_firstChild(&pChild);	
	while (pChild != NULL) {
		pChild->get_nodeName(&nodeName);
		wcstombs_s(NULL, chName, MAX_STRING_SIZE, nodeName, MAX_STRING_SIZE);

		if (strcmp(chName, "condition") == 0)
			pEvent	= GetCondition(pChild);
		else if (strcmp(chName, "proActuator") == 0) 
			pAct	= GetActuator(pChild);
		else if (strcmp(chName, "reActuator") == 0) {
			pSyncAct= GetActuator(pChild);
		}
		else check(false, _T("XML Rule Error"));

		pChild->get_nextSibling(&pNext);
		pChild = pNext;
	}
	m_pMetaObject->AddObject(pEvent, pAct, pSyncAct);
}

/*****************************************************************************************************/
// protected 2:
CAwareToolPtr CAwareXMLParser::CreateAwareTool(char * chLib)
{
	CAwareToolPtr pAwareTool;
	HINSTANCE hinstLib;

	hinstLib = LoadLibraryA(chLib);
	check(hinstLib != NULL, _T("dll fail"));

	PAWARETOOL pGetAware = (PAWARETOOL)GetProcAddress(hinstLib, "GetThisComponent");
	if (pGetAware == NULL)	{
		// handle the error
		AfxMessageBox(_T("func fail"), MB_OK, 0);
		FreeLibrary(hinstLib);
	}
	else {
		pAwareTool = pGetAware(); // call the function
		pAwareTool->SetDllInst(hinstLib);
	}
	return pAwareTool;
}

CAwareletPtr CAwareXMLParser::CreateAwarelet(char * chLib)
{
	CAwareletPtr pCompn;
	HINSTANCE hinstLib;

	hinstLib = LoadLibraryA(chLib);
	check(hinstLib != NULL, _T("dll fail"));

	PAWARELET pGetCompn = (PAWARELET)GetProcAddress(hinstLib, "GetThisComponent");
	if (pGetCompn == NULL) {
		// handle the error
		AfxMessageBox(_T("func fail"), MB_OK, 0);
		FreeLibrary(hinstLib);
	}
	else {
		// call the function
		pCompn = pGetCompn();
		pCompn->SetDllInst(hinstLib);
	}

	return pCompn;
}

CAwareSrcPtr CAwareXMLParser::GetSingleAwareTool(IXMLDOMNodePtr pAware)
{
	CAwareSrcPtr thisAware = new CAwareSource();
	check(thisAware!=NULL, _T("Create Awareness Error!"));

	// parse attributes
	IXMLDOMNamedNodeMapPtr attrMap;
	pAware->get_attributes(&attrMap);

	long len;
	BSTR attrName, attrValue;
	char chAttrName[MAX_STRING_SIZE], chAttrValue[MAX_STRING_SIZE],
		chToolName[MAX_STRING_SIZE], chToolVersion[MAX_STRING_SIZE];
	IXMLDOMNodePtr attrItem;

	attrMap->get_length(&len);
	for (long i=0; i<len; i++) {
		attrMap->get_item(i, &attrItem);
		attrItem->get_nodeName(&attrName);
		wcstombs_s(NULL, chAttrName, MAX_STRING_SIZE, attrName, MAX_STRING_SIZE);

		if (strcmp(chAttrName, "name") == 0) {
			attrItem->get_text(&attrValue);
			wcstombs_s(NULL, chToolName, MAX_STRING_SIZE, attrValue, MAX_STRING_SIZE);
		}
		else if (strcmp(chAttrName, "version") == 0) {
			attrItem->get_text(&attrValue);
			wcstombs_s(NULL, chToolVersion, MAX_STRING_SIZE, attrValue, MAX_STRING_SIZE);
		}
	}
	char * compnt = m_pCMClient->GetComponent(chToolName, (float)atof(chToolVersion));
	thisAware->SetAwareTool(CreateAwareTool(compnt));
	free(compnt);


	// parse subsets
	IXMLDOMNodePtr pChild, pNext;
	pAware->get_firstChild(&pChild);

	while (pChild != NULL) {
		pChild->get_nodeName(&attrName);
		wcstombs_s(NULL, chAttrName, MAX_STRING_SIZE, attrName, MAX_STRING_SIZE);

		if (strcmp(chAttrName, "alias") == 0){
			pChild->get_text(&attrValue);
			wcstombs_s(NULL, chAttrValue, MAX_STRING_SIZE, attrValue, MAX_STRING_SIZE);
			thisAware->SetName(chAttrValue);
		}
		else if (strcmp(chAttrName, "param") == 0) {
			pChild->get_attributes(&attrMap);
			attrMap->get_item(0, &attrItem);
			attrItem->get_text(&attrName);
			pChild->get_text(&attrValue);

			char key[MAX_STRING_SIZE], value[MAX_STRING_SIZE];
			wcstombs_s(NULL, key, MAX_STRING_SIZE, attrName, MAX_STRING_SIZE);
			wcstombs_s(NULL, value, MAX_STRING_SIZE, attrValue, MAX_STRING_SIZE);

			thisAware->SetConfParam(key, value);
		}
		else check(false, _T("XML AwareTool Error"));

		pChild->get_nextSibling(&pNext);
		pChild = pNext;
	}
	return thisAware;
}

CRecfnCompntPtr CAwareXMLParser::GetSingleAwarelet(IXMLDOMNodePtr pCompn, AWARELETTYPE &type)
{
	CRecfnCompntPtr awarelet = new CReconfigComponent();

	// parse attributes
	IXMLDOMNamedNodeMapPtr attrMap;
	pCompn->get_attributes(&attrMap);

	long len;
	BSTR attrName, attrValue;
	char chAttrName[MAX_STRING_SIZE], chAttrValue[MAX_STRING_SIZE], 
		chletName[MAX_STRING_SIZE], chletVersion[MAX_STRING_SIZE];
	IXMLDOMNodePtr attrItem;

	attrMap->get_length(&len);
	for (long i=0; i<len; i++) {
		attrMap->get_item(i, &attrItem);
		attrItem->get_nodeName(&attrName);
		wcstombs_s(NULL, chAttrName, MAX_STRING_SIZE, attrName, MAX_STRING_SIZE);

		if (strcmp(chAttrName, "name") == 0) {
			attrItem->get_text(&attrValue);
			wcstombs_s(NULL, chletName, MAX_STRING_SIZE, attrValue, MAX_STRING_SIZE);
		}
		else if (strcmp(chAttrName, "version") == 0) {
			attrItem->get_text(&attrValue);
			wcstombs_s(NULL, chletVersion, MAX_STRING_SIZE, attrValue, MAX_STRING_SIZE);
		}
		else if (strcmp(chAttrName, "type") == 0) {
			attrItem->get_text(&attrValue);
			wcstombs_s(NULL, chAttrValue, MAX_STRING_SIZE, attrValue, MAX_STRING_SIZE);
			if (strcmp(chAttrValue, "proActive") == 0)  type = ACTIVE;
			else if (strcmp(chAttrValue, "reActive") == 0) type = REACTIVE;
			else if (strcmp(chAttrValue, "Both") == 0) type = BOTH;
			else check(false, _T("\nNo this type of Component\n"));
		}
	}
	char * compnt = m_pCMClient->GetComponent(chletName, (float)atof(chletVersion));
	awarelet->SetComponent(CreateAwarelet(compnt));
	free(compnt);


	// parse subsets
	IXMLDOMNodePtr pChild, pNext;
	pCompn->get_firstChild(&pChild);
	while (pChild != NULL) {
		pChild->get_nodeName(&attrName);
		wcstombs_s(NULL, chAttrName, MAX_STRING_SIZE, attrName, MAX_STRING_SIZE);

		if (strcmp(chAttrName, "alias") == 0){
			pChild->get_text(&attrValue);
			wcstombs_s(NULL, chAttrValue, MAX_STRING_SIZE, attrValue, MAX_STRING_SIZE);
			awarelet->SetName(chAttrValue);
		}
		else if (strcmp(chAttrName, "param") == 0) {
			pChild->get_attributes(&attrMap);
			attrMap->get_item(0, &attrItem);
			attrItem->get_text(&attrName);
			pChild->get_text(&attrValue);

			char key[MAX_STRING_SIZE], value[MAX_STRING_SIZE];
			wcstombs_s(NULL, key, MAX_STRING_SIZE, attrName, MAX_STRING_SIZE);
			wcstombs_s(NULL, value, MAX_STRING_SIZE, attrValue, MAX_STRING_SIZE);

			awarelet->AddParam(key, value);
		}
		else check(false, _T("XML Component Error"));

		pChild->get_nextSibling(&pNext);
		pChild = pNext;
	}

	return awarelet;
}

CEventPtr CAwareXMLParser::GetCondition(IXMLDOMNodePtr pCond)
{
	CEventPtr pawareEvent = NULL;

	IXMLDOMNodePtr pEvent;
	pCond->get_firstChild(&pEvent);	

	BSTR nodeName;
	pEvent->get_nodeName(&nodeName);
	char chName[MAX_STRING_SIZE];
	wcstombs_s(NULL, chName, MAX_STRING_SIZE, nodeName, MAX_STRING_SIZE);

	IXMLDOMNamedNodeMapPtr attrMap;
	pEvent->get_attributes(&attrMap);
	IXMLDOMNodePtr attrItem;
	attrMap->get_item(0, &attrItem);
	BSTR attrValue;
	attrItem->get_text(&attrValue);
	char chOp[MAX_STRING_SIZE];
	wcstombs_s(NULL, chOp, MAX_STRING_SIZE, attrValue, MAX_STRING_SIZE);

	if (strcmp(chName, "boolean") == 0) {
		// compositional event, recursive to parse the structure
		pawareEvent = new CCompositionalEvent();

		IXMLDOMNodePtr plhs, prhs;
		pEvent->get_firstChild(&plhs);
		plhs->get_nextSibling(&prhs);

		//set the left source
		CEventPtr plEvent = GetCondition(plhs);
		pawareEvent->SetSource(CEventSource::LEFTSOURCE, plEvent);
		//set the right source
		CEventPtr prEvent = GetCondition(prhs);
		pawareEvent->SetSource(CEventSource::RIGHTSOURCE, prEvent);

		CBooleanConditionerPtr pBCondt = new CBooleanConditioner(m_cEventTag++);
		if (strcmp(chOp, "AND") == 0) {
			pBCondt->SetType(CEventConditioner::AND);
		}
		else if (strcmp(chOp, "OR") == 0) {
			pBCondt->SetType(CEventConditioner::OR);
		}
		else check(false, _T("XML Boolean Operation Error"));

		pawareEvent->SetConditioner(pBCondt);

		plEvent->SetListener(pBCondt->GetCondtEvent(CEventSource::LEFTSOURCE));
		prEvent->SetListener(pBCondt->GetCondtEvent(CEventSource::RIGHTSOURCE));

	}
	else if (strcmp(chName, "comparison") == 0) {
		// simple event, set the source, conditioner and listener
		pawareEvent = new CSimpleEvent();
		CCompareConditionerPtr pCCondt = new CCompareConditioner(m_cEventTag++);

		if (strcmp(chOp, "BIGGER") == 0) {
			pCCondt->SetType(CEventConditioner::BIGGER);
		}
		else if (strcmp(chOp, "BETWEEN") == 0) {
			pCCondt->SetType(CEventConditioner::BETWEEN);
		}
		else if (strcmp(chOp, "SMALLER") == 0) {
			pCCondt->SetType(CEventConditioner::SMALLER);
		}
		else if (strcmp(chOp, "BESIDE") == 0) {
			pCCondt->SetType(CEventConditioner::BESIDE);
		}
		else check(false, _T("XML Compare Operation Error"));

		IXMLDOMNodePtr pChild, pNext;
		pEvent->get_firstChild(&pChild);
		while (pChild != NULL) {
			BSTR childNodeName;
			pChild->get_nodeName(&childNodeName);
			char chChildName[MAX_STRING_SIZE];
			wcstombs_s(NULL, chChildName, MAX_STRING_SIZE, childNodeName, MAX_STRING_SIZE);

			if (strcmp(chChildName, "awareness") == 0) {
				BSTR bsAware;
				pChild->get_text(&bsAware);
				char chAware[MAX_STRING_SIZE];
				wcstombs_s(NULL, chAware, MAX_STRING_SIZE, bsAware, MAX_STRING_SIZE);

				for (int i=0; i<m_pMetaObject->m_pAwareTools.GetSize(); i++) 
					if (strcmp(m_pMetaObject->m_pAwareTools[i]->GetName(), chAware) == 0) {
						pawareEvent->SetSource(CEventSource::AWARESOURCE, m_pMetaObject->m_pAwareTools[i]);
						break;
					}
			}
			else if (strcmp(chChildName, "lowerbound") == 0) {
				BSTR bslb;
				pChild->get_text(&bslb);
				char chlb[MAX_STRING_SIZE];
				wcstombs_s(NULL, chlb, MAX_STRING_SIZE, bslb, MAX_STRING_SIZE);				
				pCCondt->SetLowerBound(atof(chlb));
			}
			else if (strcmp(chChildName, "upperbound") == 0) {
				BSTR bsub;
				pChild->get_text(&bsub);
				char chub[MAX_STRING_SIZE];
				wcstombs_s(NULL, chub, MAX_STRING_SIZE, bsub, MAX_STRING_SIZE);				
				pCCondt->SetUpperBound(atof(chub));
			}
			else check(false, _T("XML Op Compare Error"));

			pChild->get_nextSibling(&pNext);
			pChild = pNext;
		}
		pawareEvent->SetConditioner(pCCondt);
	}

	return pawareEvent;
}

CActuatorPtr CAwareXMLParser::GetActuator(IXMLDOMNodePtr pActs)
{
	CActuatorPtr pActuator = new CAwareActuator();

	IXMLDOMNodePtr pChild, pNext;
	pActs->get_firstChild(&pChild);	
	
	while (pChild != NULL) {
		pActuator->AddCompt(GetSingleAct(pChild));

		pChild->get_nextSibling(&pNext);
		pChild = pNext;
	}
	return pActuator;
}

CRecfnCompntPtr CAwareXMLParser::GetSingleAct(IXMLDOMNodePtr pAct)
{
	CRecfnCompntPtr actComp = new CReconfigComponent();

	IXMLDOMNamedNodeMapPtr attrMap;
	pAct->get_attributes(&attrMap);

	IXMLDOMNodePtr attrItem;
	attrMap->get_item(0, &attrItem);

	BSTR attrName, attrValue;
	attrItem->get_text(&attrName);
	char chAttrName[MAX_STRING_SIZE];
	wcstombs_s(NULL, chAttrName, MAX_STRING_SIZE, attrName, MAX_STRING_SIZE);
	actComp->SetName(chAttrName);

	IXMLDOMNodePtr pChild, pNext;
	pAct->get_firstChild(&pChild);
	while (pChild != NULL) {
		pChild->get_attributes(&attrMap);
		if (attrMap == NULL) break;

		attrMap->get_item(0, &attrItem);
		attrItem->get_text(&attrName);

		pChild->get_text(&attrValue);

		char key[MAX_STRING_SIZE], value[MAX_STRING_SIZE];
		wcstombs_s(NULL, key, MAX_STRING_SIZE, attrName, MAX_STRING_SIZE);
		wcstombs_s(NULL, value, MAX_STRING_SIZE, attrValue, MAX_STRING_SIZE);

		actComp->SetParam(key, value);

		pChild->get_nextSibling(&pNext);
		pChild = pNext;
	}

	return actComp;
}

int	CAwareXMLParser::getChildrenNum(IXMLDOMNodePtr pNode)
{
	int num = 0;
	IXMLDOMNodePtr pChild, pNext;
	
	pNode->get_firstChild(&pChild);
	while (pChild != NULL) {
		pChild->get_nextSibling(&pNext);
		pChild = pNext;
		num ++;
	}
	return num;
}

void CAwareXMLParser::dump_com_error(_com_error &e)
{
	CString strErr;
	strErr.Format(_T("Error\n\a\tCode = %08lx\n\a\tCode meaning = "), e.Error());
	strErr += e.ErrorMessage();
	strErr += "\a\tSource = ";
	_bstr_t bstrSource(e.Source());
	_bstr_t bstrDescription(e.Description());
	strErr += (LPCTSTR)bstrSource;
	strErr += "\n\a\tDescription = ";
	strErr += (LPCTSTR)bstrDescription;
	strErr += "\n";
	MessageBox(NULL, strErr, _T("MSXML Error"), MB_OK);
}

void CAwareXMLParser::check(bool b, LPTSTR msg)
{
	if (!b) {
		MessageBox(NULL, msg, _T("XML Parsing Error"), MB_OK);
		exit(0);
	}
}

