/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
/*********************************************************
Component.
This is the parent class of all reconfigurable components.
Any one can implement his(her) own components by inheriting
this class. 
The implemented components could be a DLL, so that our
middleware can load the dll dynamically, just like awareness
measurement tools
There is only one instance of each component in our middleware.
All the actuators or metaobject just get a pointer pointing to
the component.
*********************************************************/
#pragma once

#define BMPINFO_PACK_NO_COMPRESS	1
#define BMPINFO_PACK_COMPRESS		2
#define	BMPDATA_PACK_NO_COMPRESS	3
#define	BMPDATA_PACK_COMPRESS		4


#define SYNCHRONIZATION_REQUEST		255
#define SYNCHRONIZATION_RESPONSE	254
#define SYNCHRONIZATION_EXIT		253
#define	SYNCHRONIZATION_ASK			252

#define MAX_COMPONENT_SIZE			1024
#define MAX_PARAMETER_NUM			10
#define MAX_STRING_LEN				255
#define SOCKET_VIDEO_SEND_PORT		1129
#define SOCKET_VIDEO_RECV_PORT		1130
#define THREAD_END_MAXWAIT			5

#define DEFAULT_DISPLAY_WIDTH		160	//	352
#define	DEFAULT_DISPLAY_HEIGHT		120	//	288
// If we do not use compress, 
// set <COM_SUPPORTED_WIDTH == CAP_ACCEPTABLE_WIDTH>
// and <COM_SUPPORTED_HEIGHT == CAP_ACCEPTABLE_HEIGHT>
#define CAP_ACCEPTABLE_WIDTH		160	//	160	176	320	352	640	800
#define CAP_ACCEPTABLE_HEIGHT		120	//	120	144	240	288	480	600
//									CPARAM_SQCIF	CPARAM_QCIF	CPARAM_CIF	CPARAM_4CIF	CPARAM_16CIF
#define COM_SUPPORTED_WIDTH			128	//	128		176			352			704			1408
#define	COM_SUPPORTED_HEIGHT		96	//	96		144			288			576			1152

#define GRAB_APP_PARA_MAINWND		"MainWindowParam"
#define GRAB_APP_PARA_DISPWIDTH		"DisplayWidth"
#define GRAB_APP_PARA_DISPHEIGHT	"DisplayHeight"
#define GRAB_APP_PARA_AUTOCAPTURE	"AutoCapture"
#define GRAB_APP_PARA_VIDEOSOURCE	"SetVideoSource"
#define GRAB_APP_PARA_VIDEOFORMAT	"SetVideoFormat"
#define GRAB_APP_PARA_VIDEODISPLAY	"SetVideoDisplay"
#define GRAB_APP_PARA_SHOWLOCAL		"ShowVideoLocal"
#define GRAB_APP_PARA_SAVEMODE		"SaveInfoHeader"
#define SEND_APP_PARA_REMOTEADDRESS	"RemoteAddress"
#define DISP_APP_PARA_MAINWND		"MainWindowParam"
#define DISP_APP_PARA_DISPWIDTH		"DisplayWidth"
#define DISP_APP_PARA_DISPHEIGHT	"DisplayHeight"

#define GRAB_CONF_PARA_DISP_WIDTH	"DisplayWidth"
#define GRAB_CONF_PARA_DISP_HEIGHT	"DisplayHeight"
#define GRAB_CONF_PARA_CAPT_WIDTH	"CaptureWidth"
#define GRAB_CONF_PARA_CAPT_HEIGHT	"CaptureHeight"
#define GRAB_CONF_PARA_COMP_WIDTH	"CompressWidth"
#define GRAB_CONF_PARA_COMP_HEIGHT	"CompressHeight"
#define COMP_CONF_PARA_COMP_WIDTH	"CompressWidth"
#define COMP_CONF_PARA_COMP_HEIGHT	"CompressHeight"
#define DECO_CONF_PARA_COMP_WIDTH	"CompressWidth"
#define DECO_CONF_PARA_COMP_HEIGHT	"CompressHeight"
#define DISP_CONF_PARA_DISP_WIDTH	"DisplayWidth"
#define DISP_CONF_PARA_DISP_HEIGHT	"DisplayHeight"
#define DISP_CONF_PARA_COMP_WIDTH	"CompressWidth"
#define DISP_CONF_PARA_COMP_HEIGHT	"CompressHeight"

#define WIDTHBYTES(bits)    (((bits) + 31) / 32 * 4) 

#include <Afxtempl.h> // array
#include "component.h"

typedef CArray<COMPNTPARAM, COMPNTPARAM&> COMPNTPARAMArray;

class CAwarelet : public CComponent
{
public:
	CAwarelet(void) {
		input_data		= NULL;
		input_length	= 0;
		output_data		= NULL;
		output_length	= 0;
	}
public:
	virtual ~CAwarelet(void) {
		input_data		= NULL;
		input_length	= 0;
		if (output_data	!= NULL) {
			free(output_data);
			output_data = NULL;
			output_length = 0;
		}
	}
public:
	bool Input(void * buf, size_t len) {
		input_data		= buf;
		input_length	= len;
		return true;
	}
	bool Output(void ** buf, size_t * len) {
		if (buf!=NULL) *buf = output_data;
		if (len!=NULL) *len	= output_length;
		return true;
	}

protected:
	void * input_data;
	size_t input_length;
	void * output_data;
	size_t output_length;
};

typedef CAwarelet * (WINAPI * PAWARELET) ();
typedef CAwarelet * CAwareletPtr;
typedef CArray<CAwareletPtr, CAwareletPtr&> CAwareletPtrArray;
