/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"
#include "BooleanConditioner.h"

CBooleanConditioner::CBooleanConditioner(void)
{
	strcpy_s(m_strLEventTag, "LBooleanConditionerEvent\0");
	strcpy_s(m_strREventTag, "RBooleanConditionerEvent\0");
	m_hSourceEvent[0] = CreateEventA(NULL,FALSE,FALSE,m_strLEventTag);
	m_hSourceEvent[1] = CreateEventA(NULL,FALSE,FALSE,m_strREventTag);
	m_threadSourceWaiting = NULL;
	m_statusSourceWaiting = CLOSED;
	m_pLeftSource	= NULL;
	m_pRightSource	= NULL;
}

CBooleanConditioner::CBooleanConditioner(char chTag)
{
	strcpy_s(m_strLEventTag, "LBooleanConditionerEvent\0");
	strcpy_s(m_strREventTag, "RBooleanConditionerEvent\0");
	size_t len = strlen(m_strLEventTag);
	m_strLEventTag[len] = chTag;
	m_strLEventTag[len+1] = '\0';
	len = strlen(m_strREventTag);
	m_strREventTag[len] = chTag;
	m_strREventTag[len+1] = '\0';
	m_hSourceEvent[0] = CreateEventA(NULL,FALSE,FALSE,m_strLEventTag);
	m_hSourceEvent[1] = CreateEventA(NULL,FALSE,FALSE,m_strREventTag);
	m_threadSourceWaiting = NULL;
	m_statusSourceWaiting = CLOSED;
	m_pLeftSource	= NULL;
	m_pRightSource	= NULL;
}

CBooleanConditioner::~CBooleanConditioner(void)
{
	CloseHandle(m_hSourceEvent[0]);
	CloseHandle(m_hSourceEvent[1]);
	if (m_threadSourceWaiting != NULL) {
		StopThread();
	}
	m_pLeftSource	= NULL;
	m_pRightSource	= NULL;
}

void CBooleanConditioner::SetType(CONDTYPE type)
{
	m_condType = type;
}

void CBooleanConditioner::SetEventSource(CEventSource::SOURCETYPE type, CEventSource * pSrc)
{
	switch (type) {
	case CEventSource::LEFTSOURCE:
		m_pLeftSource = (CAwareEvent*)pSrc;
		break;
	case CEventSource::RIGHTSOURCE:
		m_pRightSource = (CAwareEvent*)pSrc;
		break;
	default:
		m_pLeftSource = NULL;
		m_pRightSource= NULL;
	}
}

HANDLE CBooleanConditioner::GetCondtEvent(CEventSource::SOURCETYPE type)
{
	if (type == CEventSource::LEFTSOURCE)
		return m_hSourceEvent[0];
	else if (type == CEventSource::RIGHTSOURCE)
		return m_hSourceEvent[1];
	return NULL;
}

void CBooleanConditioner::Start()
{
	StartThread();
}

void CBooleanConditioner::StartThread(){
	m_threadSourceWaiting = CreateThread(NULL, 0, ThreadSrcWaiting, this, 0, NULL);
	m_statusSourceWaiting = RUNNING;
	return;
}

DWORD WINAPI CBooleanConditioner::ThreadSrcWaiting(LPVOID arg)
{
	CBooleanConditioner * thisClass = (CBooleanConditioner *)arg;
	thisClass->SrcWaitingFun();
	return 0;
}

void CBooleanConditioner::SrcWaitingFun()
{
	DWORD dwEvent;
	bool bleft=false, bright=false;
	SYSTEMTIME time1, time2;
	GetSystemTime(&time1);
	while(m_statusSourceWaiting == RUNNING) {
		dwEvent = WaitForMultipleObjects(2, m_hSourceEvent, FALSE, INFINITE);
		if (m_statusSourceWaiting != RUNNING) break;
		GetSystemTime(&time2);
		if (time2.wMilliseconds-time1.wMilliseconds<500 && time2.wSecond == time1.wSecond &&
			time2.wMinute == time1.wMinute && time2.wHour == time1.wHour) break;
		time2 = time1;

		switch (dwEvent) { 
		case WAIT_OBJECT_0 + 0: 
			bleft = true;
			bright= getSourceData(CEventSource::RIGHTSOURCE);
			break; 
		case WAIT_OBJECT_0 + 1: 
			bright= true;
			bleft = getSourceData(CEventSource::LEFTSOURCE);
			break; 
		}

		bool m_bCondtResult = false;
		switch(m_condType) {
		case AND:
			m_bCondtResult = bleft && bright;
			break;
		case OR:
			m_bCondtResult = bleft || bright;
			break;
		}
		
		if (m_bCondtResult) SetEvent(m_hListenerEvent);
		
		//OnReceive();
	}

	m_statusSourceWaiting = CLOSING;

	// do something else;

	// Self terminate the thread
	m_statusSourceWaiting = CLOSED;
	m_threadSourceWaiting = NULL;
}

void CBooleanConditioner::Stop()
{
	StopThread();
}

void CBooleanConditioner::StopThread()
{
	int watchDog = 0;
	while (m_threadSourceWaiting)
	{
		m_statusSourceWaiting = CLOSING;
		SetEvent(m_hSourceEvent[0]);
		SetThreadPriority(m_threadSourceWaiting, THREAD_PRIORITY_HIGHEST);
		if((::WaitForSingleObject(m_threadSourceWaiting, 1000) != WAIT_TIMEOUT) ||
			watchDog >= THREAD_TERMINATION_MAXWAIT)
		{
			CloseHandle(m_threadSourceWaiting);
			m_statusSourceWaiting = CLOSED;
			m_threadSourceWaiting = NULL;
		}
		watchDog++;
	}
}

bool CBooleanConditioner::getSourceData(CEventSource::SOURCETYPE side)
{
	char chValue[MAX_VALUE_SIZE];
	int len;
	bool bready;
	if (side == CEventSource::LEFTSOURCE) 
		bready = m_pLeftSource->GetValue(chValue, &len);
	else if (side == CEventSource::RIGHTSOURCE)
		bready = m_pRightSource->GetValue(chValue, &len);
	else return false;

	if (atoi(chValue) > 0 && bready) return true;
	return false;
}

bool CBooleanConditioner::GetCondtResult(bool bFormat)
{
	return getSourceData(CEventSource::LEFTSOURCE) && getSourceData(CEventSource::RIGHTSOURCE);
}