/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"
#include "CM.h"
#include "IniFile.h"
#include "Component.h"

#define CONFIG_FILE_NAME	"cm.inf"
#define CONFIG_AREA_NAME	"Components"
#define CONFIG_EXTT_NAME	"ext"
#define CONFIG_DIRY_NAME	"dir"

CCM::CCM(void)
{
	m_bIsStarted = false;
	status_thread = CLOSED;
	status_socket = DISCONNECTED;
}

CCM::~CCM(void)
{
	if (m_bIsStarted) Stop();
}

bool CCM::Start()
{
	if (m_bIsStarted) return true;
	
	CreateComponentTable();
	StartWaitingThread();
	
	m_bIsStarted = true;
	return true;
}

bool CCM::Stop()
{
	if (!m_bIsStarted) return true;

	StopWaitingThread();
	ReleaseComponentTable();

	m_bIsStarted = false;
	return true;
}

bool CCM::IsStarted()
{
	return m_bIsStarted;
}

bool CCM::CreateComponentTable()
{
	// Load and parse Inifile, get the extension and directory information
	CIniFile inif;
	inif.IniOpen(CONFIG_FILE_NAME);

	char content[1024];

	inif.ReadIni(CONFIG_AREA_NAME, CONFIG_EXTT_NAME, content);
	int extNum = GetStringNum(content);
	CString * extList = new CString[extNum];
	GetStringList(content, extNum, extList);

	inif.ReadIni(CONFIG_AREA_NAME, CONFIG_DIRY_NAME, content);
	int dirNum = GetStringNum(content);
	CString * dirList = new CString[dirNum];
	GetStringList(content, dirNum, dirList);

	inif.IniClose();

	WIN32_FIND_DATA FindFileData;
	HANDLE hFind = INVALID_HANDLE_VALUE;
	DWORD dwError;
	CString strFileName;
	CCMId cid;
	char * addr;

	for (int i=0; i<dirNum; i++) {
		for (int j=0; j<extNum; j++) {
			CString extFile = dirList[i] + _T("\\*.") + extList[j];
			hFind = FindFirstFile(extFile, &FindFileData);
			if (INVALID_HANDLE_VALUE == hFind) continue;

			// List all the files in the directory.
			do {
				strFileName = dirList[i] + _T("\\") + FindFileData.cFileName;
				addr = (char *)malloc(strFileName.GetLength() + 1);
				wcstombs_s(NULL, addr, strFileName.GetLength()+1, strFileName, strFileName.GetLength()+1);

				if (GetCMId(addr, cid)) {
					if (tb.has(cid)) tb[cid] = addr;
					else tb.add(cid, addr);
				}
				else AfxMessageBox(strFileName + _T(" is not a component"));
			} while(FindNextFile(hFind, &FindFileData) != 0);
	

			dwError = GetLastError();
			FindClose(hFind);
			check(dwError == ERROR_NO_MORE_FILES, _T("FindNextFile error"));
		}
	}
	return true;
}

bool CCM::StartWaitingThread()
{
	StartNetwork();
	CreateSocket();
	m_threadWaiting = CreateThread(NULL, 0, EventWaitingThread, this, 0, NULL);
	status_thread	= RUNNING;
	return true;
}

bool CCM::StartNetwork()
{
	WSADATA wsaData;
	WORD wVersionRequested = MAKEWORD(2,2);
	int nRet;

	// Init WinSock
	nRet = WSAStartup(wVersionRequested, &wsaData);
	if (nRet != 0) {
		AfxMessageBox(_T("\nError initializing WinSock\n"));
		return FALSE;
	}

	// Check version
	if (wsaData.wVersion != wVersionRequested) {
		AfxMessageBox(_T("\nWinSock version not supported\n"));
		WSACleanup();
		return FALSE;
	}

	status_socket = INITIALIZED;
	return TRUE;
}

bool CCM::CreateSocket()
{
	if (status_socket < INITIALIZED) StartNetwork();
	m_sock = socket(AF_INET, SOCK_DGRAM, 0);
	if (m_sock == INVALID_SOCKET) {
		AfxMessageBox(_T("Socket Error: Can not create Socket!"));
		CloseNetwork();
		return FALSE;
	}
	status_socket = CREATED;

	//bind local address to socket
	memset(&m_localAddr,0,sizeof(m_localAddr));
	m_localAddr.sin_family = AF_INET;
	m_localAddr.sin_addr.s_addr = htonl(INADDR_ANY);
	m_localAddr.sin_port = htons(CM_SERVER_PORT);
	int nb = bind(m_sock,(struct sockaddr *)&m_localAddr,sizeof(m_localAddr));
	if (nb == SOCKET_ERROR) {
		AfxMessageBox(_T("Bind Error: Can not bind local address to Socket!"));
		CloseNetwork();
		return FALSE;
	}

	// set remote host information. Could be broadcast
	return TRUE;
}

DWORD WINAPI CCM::EventWaitingThread(LPVOID arg)
{
	CCM * thisClass = (CCM *)arg;
	thisClass->EventWaitingFun();
	return 0;
}

void CCM::EventWaitingFun()
{
	char buff[MAX_STRING_SIZE];

	struct sockaddr recvaddr;
	int rcvAddrLen = sizeof(recvaddr);
	int rcvLen;

	while(status_thread == RUNNING) {
		rcvLen = recvfrom(m_sock, buff, MAX_STRING_SIZE, 0, (sockaddr *)&recvaddr, &rcvAddrLen);
		if (status_thread != RUNNING) break;
		if (rcvLen ==SOCKET_ERROR || rcvLen <=0) break;
		
		CCMId cid;
		cid.fromString(buff);
		strcpy_s((char*)buff, MAX_STRING_SIZE, tb[cid]);
		sendto(m_sock, buff, int(strlen((char*)buff)+1), 0, (sockaddr *)&recvaddr, sizeof(recvaddr));
	}

	status_thread = CLOSING;

	if(status_socket > DISCONNECTED)	CloseNetwork();

	// Self terminate the thread
	m_threadWaiting = NULL;
	status_thread = CLOSED;
}

void CCM::StopWaitingThread()
{
	CloseNetwork();

	int watchDog = 0;
	while (m_threadWaiting)
	{
		status_thread = CLOSING;
		SetThreadPriority(m_threadWaiting, THREAD_PRIORITY_HIGHEST);
		if((::WaitForSingleObject(m_threadWaiting, 1000) != WAIT_TIMEOUT) ||
			watchDog >= THREAD_END_MAXWAIT)
		{
			CloseHandle(m_threadWaiting);
			m_threadWaiting = NULL;
			status_thread = CLOSED;
		}
		watchDog++;
	}
}

void CCM::CloseNetwork()
{
	if (status_socket > INITIALIZED) CloseSocket();
	if (status_socket > DISCONNECTED) {
		WSACleanup();
		status_socket = DISCONNECTED;
	}
}

void CCM::CloseSocket()
{
	if (status_socket >= CREATED) {
		shutdown(m_sock, SD_BOTH);
		closesocket(m_sock);
		status_socket = INITIALIZED;
	}
}

void CCM::ReleaseComponentTable()
{
	tb.iteratorReset();
	while (!tb.atEnd()) free(tb[tb.nextKey()]);
}

int CCM::GetStringNum(char * content)
{
	int num = 1;
	for (int i=0; content[i]!='\0'; i++) 
		if ('|'==content[i]) num++;
	return num;
}

void CCM::GetStringList(char * content, int num, CString list[])
{
	CString str, strCont;
	strCont = content;
	for (int pos, i=0; i<num-1; i++) {
		pos = strCont.Find('|');
		if (-1 == pos) break;
		list[i] = strCont.Left(pos);
		list[i].Trim();
		strCont = strCont.Right(strCont.GetLength() - pos - 1);
	}
	list[num-1] = strCont;
	list[num-1].Trim();
}


bool CCM::GetCMId(char * fname, CCMId &id)
{
	HINSTANCE hinstLib = LoadLibraryA(fname);
	if (hinstLib == NULL) return false;

	PCOMPONENT pGetCompn = (PCOMPONENT)GetProcAddress(hinstLib, "GetThisComponent");
	if (pGetCompn == NULL) {
		FreeLibrary(hinstLib);
		return false;
	}

	// call the function
	CCompnPtr pCompn = pGetCompn();
	id = pCompn->GetCId();

	delete pCompn;
	FreeLibrary(hinstLib);

	return true;
}

void CCM::check(bool b, CString msg)
{
	if (!b) {
		AfxMessageBox(msg);
		exit(0);
	}
}


