/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"
#include "CMClient.h"

CCMClient::CCMClient(void)
{
	status_socket = DISCONNECTED;
	StartNetwork();
	CreateSocket();
}

CCMClient::~CCMClient(void)
{
	CloseSocket();
	CloseNetwork();
}

void CCMClient::SetRemoteAddr(char * ip, int port)
{
	u_long addr;
	LPHOSTENT remoteHost;

	if (isalpha(ip[0]))   /* host address is a name */
		remoteHost = gethostbyname(ip);
	else {
		addr = inet_addr(ip);
		remoteHost = gethostbyaddr((char *)&addr, 4, AF_INET);
	}
	if (remoteHost == NULL) {
		AfxMessageBox(_T("GetHost Error: Destination Host not found!"), MB_OK, 0);
		CloseSocket();
		return;
	}		

	memset(&m_remoteAddr,0,sizeof(m_remoteAddr));
	m_remoteAddr.sin_family = AF_INET;
	memcpy((char *)&m_remoteAddr.sin_addr.s_addr, remoteHost->h_addr_list[0],remoteHost->h_length);
	m_remoteAddr.sin_port = htons(port);

	status_socket = CONNECTED;
}

char * CCMClient::GetComponent(char * name, float fv)
{
	CCMId cid(name, fv);
	char * buff = (char *)malloc(MAX_STRING_SIZE);
	cid.toString(buff);
	sendto(m_sock, buff, int(sizeof(fv)+strlen(name)+1), 0, (sockaddr *)&m_remoteAddr, sizeof(m_remoteAddr));

	struct sockaddr recvaddr;
	int rcvAddrLen = sizeof(recvaddr);
	recvfrom(m_sock, buff, MAX_STRING_SIZE, 0, (sockaddr *)&recvaddr, &rcvAddrLen);
	return buff;
}

bool CCMClient::StartNetwork()
{
	WSADATA wsaData;
	WORD wVersionRequested = MAKEWORD(2,2);
	int nRet;

	// Init WinSock
	nRet = WSAStartup(wVersionRequested, &wsaData);
	if (nRet != 0) {
		AfxMessageBox(_T("\nError initializing WinSock\n"));
		return false;
	}

	// Check version
	if (wsaData.wVersion != wVersionRequested) {
		AfxMessageBox(_T("\nWinSock version not supported\n"));
		WSACleanup();
		return false;
	}

	status_socket = INITIALIZED;
	return true;
}

bool CCMClient::CreateSocket()
{
	if (status_socket < INITIALIZED) StartNetwork();

	m_sock = socket(AF_INET, SOCK_DGRAM, 0);
	if (m_sock == INVALID_SOCKET) {
		AfxMessageBox(_T("Socket Error: Can not create Socket!"));
		CloseNetwork();
		return false;
	}

	status_socket = CREATED;

	//bind local address to socket
	memset(&m_localAddr,0,sizeof(m_localAddr));
	m_localAddr.sin_family = AF_INET;
	m_localAddr.sin_addr.s_addr = htonl(INADDR_ANY);
	m_localAddr.sin_port = htons(CM_CLIENT_PORT);
	int nb = bind(m_sock,(struct sockaddr *)&m_localAddr,sizeof(m_localAddr));
	if (nb == SOCKET_ERROR) {
		AfxMessageBox(_T("Bind Error: Can not bind local address to Socket!"));
		CloseNetwork();
		return false;
	}

	return true;
}

void CCMClient::CloseSocket()
{
	if (status_socket >= CREATED) {
		shutdown(m_sock, SD_BOTH);
		closesocket(m_sock);
		status_socket = INITIALIZED;
	}
}

void CCMClient::CloseNetwork()
{
	if (status_socket > INITIALIZED) CloseSocket();
	if (status_socket > DISCONNECTED) {
		WSACleanup();
		status_socket = DISCONNECTED;
	}
}
