/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"
#include "CompareConditioner.h"

CCompareConditioner::CCompareConditioner(void)
{
	strcpy_s(m_strEventTag, "CompareConditionerEvent\0");
	m_hSourceEvent = CreateEventA(NULL,FALSE,FALSE,m_strEventTag);
	m_threadSourceWaiting = NULL;
	m_statusSourceWaiting = CLOSED;
	m_pSource = NULL;
}

CCompareConditioner::CCompareConditioner(char chTag)
{
	strcpy_s(m_strEventTag, "CompareConditionerEvent\0");
	size_t len = strlen(m_strEventTag);
	m_strEventTag[len] = chTag;
	m_strEventTag[len+1] = '\0';
	
	m_hSourceEvent = CreateEventA(NULL,FALSE,FALSE,m_strEventTag);
	m_threadSourceWaiting = NULL;
	m_statusSourceWaiting = CLOSED;
	m_pSource = NULL;
}

CCompareConditioner::~CCompareConditioner(void)
{
	CloseHandle(m_hSourceEvent);
	if (m_threadSourceWaiting != NULL) {
		StopThread();
	}
	m_pSource = NULL;
}

void CCompareConditioner::SetType(CONDTYPE type)
{
	m_condType = type;
}

void CCompareConditioner::SetEventSource(CEventSource::SOURCETYPE type, CEventSource * pSrc)
{
	if (type == CEventSource::AWARESOURCE)
		m_pSource = (CAwareSource *)pSrc;
	else m_pSource = NULL;
}

HANDLE CCompareConditioner::GetCondtEvent(CEventSource::SOURCETYPE type)
{
	if (type == CEventSource::AWARESOURCE)
		return m_hSourceEvent;
	return NULL;
}

void CCompareConditioner::SetLowerBound(double lb)
{
	m_nLowerbound = lb;
}

void CCompareConditioner::SetUpperBound(double ub)
{
	m_nUpperbound = ub;
}

void CCompareConditioner::Start()
{
	StartThread();
}

void CCompareConditioner::StartThread(){
	m_threadSourceWaiting = CreateThread(NULL, 0, ThreadSrcWaiting, this, 0, NULL);
	m_statusSourceWaiting = RUNNING;
	return;
}

DWORD WINAPI CCompareConditioner::ThreadSrcWaiting(LPVOID arg)
{
	CCompareConditioner * thisClass = (CCompareConditioner *)arg;
	thisClass->SrcWaitingFun();
	return 0;
}

void CCompareConditioner::SrcWaitingFun()
{
	while(m_statusSourceWaiting == RUNNING) {
		WaitForSingleObject(m_hSourceEvent,INFINITE);
		if (m_statusSourceWaiting != RUNNING) break;

		if (GetCondtResult(true)) SetEvent(m_hListenerEvent);
		//OnReceive();
	}

	m_statusSourceWaiting = CLOSING;

	// do something else;

	// Self terminate the thread
	m_statusSourceWaiting = CLOSED;
	m_threadSourceWaiting = NULL;
}

void CCompareConditioner::Stop()
{
	StopThread();
}

void CCompareConditioner::StopThread()
{
	int watchDog = 0;
	while (m_threadSourceWaiting)
	{
		m_statusSourceWaiting = CLOSING;
		SetEvent(m_hSourceEvent);
		SetThreadPriority(m_threadSourceWaiting, THREAD_PRIORITY_HIGHEST);
		if((::WaitForSingleObject(m_threadSourceWaiting, 1000) != WAIT_TIMEOUT) ||
			watchDog >= THREAD_TERMINATION_MAXWAIT)
		{
			CloseHandle(m_threadSourceWaiting);
			m_statusSourceWaiting = CLOSED;
			m_threadSourceWaiting = NULL;
		}
		watchDog++;
	}
}

bool CCompareConditioner::GetCondtResult(bool bFormat)
{
	double value;
	int len;
	m_pSource->GetValue(&value, &len);

	TRACE1("%f\n", value);

	bool m_bCondtResult	= false;
	switch(m_condType) {
	case BIGGER:
		if (value >= m_nLowerbound) m_bCondtResult = true;
		break;
	case SMALLER:
		if (value <= m_nUpperbound) m_bCondtResult = true;
		break;
	case BETWEEN:
		if (value >= m_nLowerbound && value <= m_nUpperbound) m_bCondtResult = true; 
		break;
	case BESIDE:
		if (value <= m_nLowerbound || value >= m_nUpperbound) m_bCondtResult = true;
	}
		
	return m_bCondtResult;
}
