/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"
#include "CompositionalEvent.h"

CCompositionalEvent::CCompositionalEvent(void)
{
	m_lpSrc = NULL;
	m_rpSrc = NULL;
	m_pCondt = NULL;
	m_pListener = NULL;
}

CCompositionalEvent::~CCompositionalEvent(void)
{
	if (m_lpSrc != NULL) {
		delete m_lpSrc;
		m_lpSrc = NULL;
	}
	if (m_rpSrc != NULL) {
		delete m_rpSrc;
		m_rpSrc = NULL;
	}

	if (m_pCondt != NULL) {
		delete m_pCondt;
		m_pCondt = NULL;
	}

	if (m_pListener != NULL) {
		delete m_pListener;
		m_pListener = NULL;
	}
}

void CCompositionalEvent::SetSource(SOURCETYPE type, CEventSource *pSrc)
{
	switch(type) {
	case LEFTSOURCE:
		m_lpSrc = pSrc;
		if (m_pCondt != NULL) {
			m_lpSrc->SetNotifier(m_pCondt->GetCondtEvent(CEventSource::LEFTSOURCE));
			m_pCondt->SetEventSource(CEventSource::LEFTSOURCE, m_lpSrc);
		}			
		break;
	case RIGHTSOURCE:
		m_rpSrc = pSrc;
		if (m_pCondt != NULL) {
			m_rpSrc->SetNotifier(m_pCondt->GetCondtEvent(CEventSource::RIGHTSOURCE));
			m_pCondt->SetEventSource(CEventSource::RIGHTSOURCE, m_rpSrc);
		}
		break;
	}
}

void CCompositionalEvent::SetConditioner(CEventConditioner * pCond)
{
	m_pCondt = pCond;
	if (m_lpSrc != NULL) {
		m_lpSrc->SetNotifier(m_pCondt->GetCondtEvent(CEventSource::LEFTSOURCE));
		m_pCondt->SetEventSource(CEventSource::LEFTSOURCE, m_lpSrc);
	}
	if (m_rpSrc != NULL) {
		m_rpSrc->SetNotifier(m_pCondt->GetCondtEvent(CEventSource::RIGHTSOURCE));
		m_pCondt->SetEventSource(CEventSource::RIGHTSOURCE, m_rpSrc);
	}
}

void CCompositionalEvent::SetListener(HANDLE pEvent)
{
	m_pListener = NULL; //maybe layer we need to listener
	SetNotifier(pEvent);
}

void CCompositionalEvent::Start()
{
//	m_pListener->Start();
	m_pCondt->Start();
	m_lpSrc->Start();
	m_rpSrc->Start();
}

void CCompositionalEvent::Stop()
{
	m_lpSrc->Stop();
	m_rpSrc->Stop();
	m_pCondt->Stop();
//	m_pListener->Stop();
}
