/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"
#include "DCEngine.h"

#include "AwareEvent.h"
#include "AwareActuator.h"
#include "AwareMetaObject.h"

BOOL APIENTRY DllMain( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
{
	switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH: break;
   }
   return TRUE;
}

/***********************************************************************************************/
// public:

CDCEngine::CDCEngine(void)
{
	m_pMetaObject	= new CAwareMetaObject();
	m_pXmlParser	= new CAwareXMLParser("dc.xml");
	m_bInitialized	= false;
	m_bStarted		= false;
}

CDCEngine::CDCEngine(char * xmlfile)
{
	m_pMetaObject	= new CAwareMetaObject();
	m_pXmlParser	= new CAwareXMLParser(xmlfile);
	m_bInitialized	= false;
	m_bStarted		= false;
}

CDCEngine::~CDCEngine(void)
{
	delete m_pMetaObject;
	delete m_pXmlParser;
}

void CDCEngine::SetXMLFile(char * xmlfile)
{
	m_pXmlParser->SetXmlFile(xmlfile);
}

void CDCEngine::SetHWnd(CWnd *pWnd)
{
	m_pMetaObject->SetHWnd(pWnd);
}

void CDCEngine::GetFormatValue(char * buf) 
{
	m_pMetaObject->GetFormatV(buf);
}

void CDCEngine::SetAppParam(char* cName, char* key, void * value)
{
	m_pMetaObject->SetCompntAppParam(cName, key, value);
}

void CDCEngine::SetAdaptation(bool b)
{
	if (b) Start();
	else Stop();
}

void CDCEngine::Initialize()
{
	if (m_bInitialized) return;
	m_pXmlParser->SetMetaObject(m_pMetaObject);
	m_pXmlParser->Parse();
	m_pMetaObject->Construct();
	m_bInitialized = true;
}

bool CDCEngine::Start()
{
	if (m_bStarted) return true;
	if (!m_bInitialized) Initialize();
	m_pMetaObject->Start();
	m_bStarted	= true;
	return true;
}

bool CDCEngine::Stop()
{
	if (!m_bStarted) return true;
	m_pMetaObject->Stop();
	m_bStarted	= false;
	return true;
}

void CDCEngine::Clear()
{
	m_pMetaObject->Deconstruct();
}

void CDCEngine::SetMetaObjectInput(void * data, size_t len)
{
	m_pMetaObject->SetInput(data, len);
}

void CDCEngine::StartMetaProcess() 
{
	m_pMetaObject->Process();
}

void CDCEngine::StartDataProcess(void * in, size_t inlen, void ** out, size_t * outlen)
{
	m_pMetaObject->ProcessActv(in, inlen, out, outlen);
}

void CDCEngine::GetMetaObjectOutput(void ** data, size_t * len)
{
	m_pMetaObject->GetOutput(data, len);
}

void CDCEngine::SyncTo(char * addr)
{
	m_pMetaObject->SyncTo(addr);
}

/***********************************************************************************************/
// proteced:
