/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
/*********************************************************
Decision Engine.
This is the main class of decision engine. 
First, it loads the xml file and parses it as following:
1. create the awareness tools
2. load the reconfigurable components
3. create the compositional event and actuator for each rule.
4. start the actuator and event.
*********************************************************/

#pragma once

#define DllExport __declspec( dllexport )
#define DllImport __declspec( dllimport )

#include "AwareMetaObject.h"
#include "AwareXMLParser.h"


class DllExport CDCEngine
{
public:
	CDCEngine(void);
	CDCEngine(char *);
public:
	~CDCEngine(void);
public:
	void GetFormatValue(char *);
	void SetHWnd(CWnd *pWnd);

	void SetMetaObjectInput(void *, size_t);
	void GetMetaObjectOutput(void **, size_t *);
	void SyncTo(char * addr);
	void StartMetaProcess();
	void StartDataProcess(void *in=NULL, size_t inlen=0, void **out=NULL, size_t *outlen=NULL);
	void SetXMLFile(char *);
	
	void Initialize();
	void Clear();
	bool Start();
	bool Stop();

	void SetAppParam(char*, char*, void * value  = (void *)1);
	void SetAdaptation(bool b = true);

protected:

private:
	CXMLParserPtr			m_pXmlParser;
	CMetaObjectPtr			m_pMetaObject;
	bool	m_bInitialized;
public:
	bool	m_bStarted;
};
