/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
#include "IniFile.h"
#include <windows.h>
#include <string.h>
#include <stdlib.h>

#define MAX_LINE_LENGTH		4096
#define MAX_AREA_LENGTH		256
#define MAX_ITEM_LENGTH		256

BOOL APIENTRY DllMain( HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
{
	switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH: break;
   }
   return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CIniFile construction
CIniFile::CIniFile()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
	m_bCloseFile = true;
}

CIniFile::CIniFile(char *pfilename)
{
	if (fopen_s(&m_pfile, pfilename, "r+t") == 0)
		m_bCloseFile = false;
}

CIniFile::~CIniFile()
{
	if(!m_bCloseFile) {
		fclose(m_pfile);
		m_bCloseFile = true;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Public functions
bool CIniFile::IniOpen(char *pfilename)
{
	if(!m_bCloseFile) fclose(m_pfile);
	if (fopen_s(&m_pfile, pfilename, "r+t") == 0)
		m_bCloseFile = false;
	else m_bCloseFile = true;
	return (!m_bCloseFile);
}

bool CIniFile::IniClose()
{
	if(!m_bCloseFile) {
		fclose(m_pfile);
		m_bCloseFile = true;
	}
	return true;
}


bool CIniFile::ReadIni(char * AreaName, char * ItemName, char * Content)
{
	if (m_bCloseFile) return NULL;
	char line[MAX_LINE_LENGTH], area[MAX_AREA_LENGTH], item[MAX_ITEM_LENGTH];
	
	fseek(m_pfile, 0, SEEK_SET);
	clearerr(m_pfile);
    
	bool bsearched = false;
	while (!feof(m_pfile)) {
		if (!ReadLine(line)) continue;
		if (GetArea(line, area) && strcmp(area, AreaName)==0) {
			bsearched = true;
			break;
		}
	}
	check(bsearched, "Can't find the IniFile Area!");

	while (!feof(m_pfile)) {
		if (!ReadLine(line)) continue;
		check(!IsArea(line), "Can't find the IniFile item!");

		if (GetItem(line, item) && strcmp(item, ItemName) == 0) {
			GetContent(line, Content);
			return true;
		}
	}
	return false;
}


/////////////////////////////////////////////////////////////////////////////
// The one and only CIniFile object
bool CIniFile::ReadLine(char *str)
{
	int i = 0;
	str[i] = getc(m_pfile);
	while (str[i] != '\n' && str[i] != EOF ) {
		i++;
		str[i] = getc(m_pfile);
	}
	str[i] = '\0';
	Trim(str);
	if (strlen(str) == 0) return false;
	return true;
}

bool CIniFile::GetArea(char *line, char *area)
{
	if (!IsArea(line)) return false;

	int pos = 0;
	for (int i=1; ']'!=line[i]; i++)
		area[pos++] = line[i];
	area[pos] = '\0';
	Trim(area);
	return true;
}

bool CIniFile::GetItem(char * line, char * name)
{
	if (IsArea(line)) return false;

	int pos = 0;
	for(int i=0; '='!=line[i]; i++) {
		if ('\0' == line[i]) return false;
		name[pos++] = line[i];
	}
	name[pos] = '\0';
	Trim(name);
	return true;
}

bool CIniFile::GetContent(char * line, char * content)
{
	if (IsArea(line)) return false;

	int i=0;
	while('=' != line[i]) {
		if ('\0' == line[i]) return false;
		i++;
	}

	int pos = 0;
	for (i=i+1; '\0'!=line[i]; i++)
		content[pos++] = line[i];
	content[pos] = '\0';
	Trim(content);
	return true;
}

bool CIniFile::IsArea(char *str)
{
	if ('['==str[0] && ']'==str[strlen(str)-1]) 
		return true;
	return false;
}

void CIniFile::check(bool b, char *msg)
{
	if(!b) {
		MessageBox(NULL, msg, "Error", MB_OK|MB_ICONERROR);
		exit(0);
	}
}

void CIniFile::stripWhite()
{
	char ch = getc(m_pfile);
	while(ch==' ' || ch=='\t' || ch=='\r')
		ch = getc(m_pfile);
	if(ch == ';')
		while(ch != '\n') ch = getc(m_pfile);
}

void CIniFile::Trim(char * str)
{
	TrimLeft(str);
	TrimRight(str);
}

void CIniFile::TrimLeft(char * str)
{
	int dist = 0;
	while(' '==str[dist] || '\t'==str[dist] || '\r'==str[dist] || '\n'==str[dist]) dist++;
	if (0 == dist) return;

	int len = strlen(str) - dist;
	for (int i=0; i<=len; i++)	// "<=len" contains the '\0'
		str[i] = str[i+dist];
}

void CIniFile::TrimRight(char * str)
{
	int len = strlen(str);
	while(' '==str[len-1] || '\t'==str[len-1] || '\r'==str[len-1] || '\n'==str[len-1]) len --;
	str[len] = '\0';
}
