/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"
#include "modsdefn.h"
#include "ModsClient.h"

CCriticalSection CModsClient::m_recvLock;

CModsClient::CModsClient()
{
	m_nSocketStatus		= NONE;
	m_nRecvThdStatus	= CLOSED;
	m_threadRecv		= NULL;
}

CModsClient::CModsClient(char * addr)
{
	m_nSocketStatus		= NONE;
	m_nRecvThdStatus	= CLOSED;
	m_threadRecv		= NULL;
	SetServerAddr(addr);
}

VOID CModsClient::SetServerAddr(char * addr)
{
	strcpy_s(servAddr, addr);
}

CModsClient::~CModsClient()
{
	Close();
}

void CModsClient::check(bool b, LPTSTR msg)
{
	if (!b) ErrorReport(msg);
}

void CModsClient::Start()
{
	StartNetwork();
	CreateSocket();
	StartTimers();
	CreateThreads();
}

void CModsClient::Close()
{
	StopTimers();
	CloseSocket();
	CloseNetwork();
	CloseThreads();
}

BOOL CModsClient::StartNetwork()
{
    WSADATA wsaData;
    WORD wVersionRequested = MAKEWORD(2,2);
    int nRet;

	// Init WinSock
    nRet = WSAStartup(wVersionRequested, &wsaData);
	check(nRet==0, _T("\nError initializing WinSock\n"));
	if (nRet != 0) return FALSE;

	// Check version
	check(wsaData.wVersion==wVersionRequested, _T("\nWinSock version not supported\n"));
	if (wsaData.wVersion != wVersionRequested) {
		WSACleanup();
		return FALSE;
	}

	m_nSocketStatus = DISCONNECTED;
	return TRUE;
}

VOID CModsClient::CloseNetwork()
{
	if (m_nSocketStatus >= DISCONNECTED) {
		WSACleanup();
		m_nSocketStatus = NONE;
	}
}

BOOL CModsClient::CreateSocket()
{
	m_sock = socket(AF_INET, SOCK_DGRAM, 0);
	check(m_sock>=0, _T("Socket Error: Can not create Socket!"));
	if (m_sock == INVALID_SOCKET) {
		Close();
		return FALSE;
	}
	m_nSocketStatus = CREATED;


	//bind local address to socket
	memset(&m_localAddr,0,sizeof(m_localAddr));
	m_localAddr.sin_family = AF_INET;
	m_localAddr.sin_addr.s_addr = htonl(INADDR_ANY);
	m_localAddr.sin_port = htons(MODS_CLIENT_PORT);
	int nb = bind(m_sock,(struct sockaddr *)&m_localAddr,sizeof(m_localAddr));
	check(nb == 0, _T("Bind Error: Can not bind local address to Socket!"));
	if (nb == SOCKET_ERROR) {
		Close();
		return FALSE;
	}

	// set remote host information. Could be broadcast
	memset(&m_modsServAddr,0,sizeof(m_modsServAddr));
	m_modsServAddr.sin_family = AF_INET;
	m_modsServAddr.sin_addr.s_addr = inet_addr(servAddr);
	m_modsServAddr.sin_port = htons(MODS_SERVER_PORT);

	m_nSocketStatus = CONNECTED;
	return TRUE;
}

VOID CModsClient::CloseSocket()
{
	if (m_nSocketStatus >= CREATED) {
		shutdown(m_sock, SD_BOTH);
		closesocket(m_sock);
		m_nSocketStatus = DISCONNECTED;
	}
}

BOOL CModsClient::StartTimers()
{
/*	pObject		= this;
	pTrapTimer	= SetTimer(NULL, NULL, MODS_TRAP_INTERVAL, (TIMERPROC)TrapTimerProc);
	check(pTrapTimer != 0, _T("Trap timer create error!"));
	
	if (pTrapTimer == 0) return FALSE;
*/	return TRUE;
}

VOID CModsClient::StopTimers()
{
//	KillTimer(NULL, pTrapTimer);
}

VOID CALLBACK CModsClient::TrapTimerProc(HWND hWnd, UINT uMsg, 
										 UINT idEvent, DWORD dwTime)
{
//	CModsClient *pSomeClass = (CModsClient*)pObject; // cast the void pointer
//	pSomeClass->TrapThread(); // call non-static function
}

VOID CModsClient::TrapThread()
{
}

BOOL CModsClient::CreateThreads()
{
	return CreateRecvThread();
}

VOID CModsClient::CloseThreads()
{
	CloseRecvThread();
}

BOOL CModsClient::CreateRecvThread()
{
	m_threadRecv = CreateThread(NULL, 0, ThreadRecv, this, 0, NULL);
	check(m_threadRecv!=NULL, _T("Thread Error: Can not create recv thread!"));
	if (m_threadRecv == NULL) return FALSE;

	m_nRecvThdStatus  = RUNNING;
	return TRUE;
}

VOID CModsClient::CloseRecvThread()
{
	int watchDog = 0;
	while (m_threadRecv)
	{
		m_nRecvThdStatus = CLOSING;
		SetThreadPriority(m_threadRecv, THREAD_PRIORITY_HIGHEST);
		if((::WaitForSingleObject(m_threadRecv, 1000) != WAIT_TIMEOUT) ||
			watchDog >= THREAD_END_MAXWAIT)
		{
			CloseHandle(m_threadRecv);
			m_nRecvThdStatus = CLOSED;
			m_threadRecv = NULL;
		}
		watchDog++;
	}
}

DWORD WINAPI CModsClient::ThreadRecv(LPVOID arg)
{
	CModsClient * thisClass = (CModsClient *)arg;
	thisClass->RecvThread();
	return 0;
}

VOID CModsClient::RecvThread()
{
	char buff[MODS_MAX_PACKET_SIZE];

	struct sockaddr recvaddr;
	int rcvAddrLen = sizeof(recvaddr);
	int rcvLen;

	while(m_nRecvThdStatus == RUNNING) {
		rcvLen = recvfrom(m_sock, buff, MODS_MAX_PACKET_SIZE, 0, (sockaddr *)&recvaddr, &rcvAddrLen);
		if (m_nRecvThdStatus != RUNNING) break;
		if (rcvLen ==SOCKET_ERROR || rcvLen <=0) break;
		buff[rcvLen] = '\0';
		
		CSingleLock csl(&CModsClient::m_recvLock);
		csl.Lock();	//ENTER CRITICAL REGION
	
		ProcBuf(buff, rcvLen);

		csl.Unlock();
		OnReceive();
	}

	m_nRecvThdStatus = CLOSING;

	if(m_nSocketStatus != NONE) {
		CloseSocket();
		CloseNetwork();
	}

	// Self terminate the thread
	m_nRecvThdStatus = CLOSED;
	m_threadRecv = NULL;
}

VOID CModsClient::SendData(char * buf, int len) {
	if (m_nSocketStatus >= CONNECTED) {
		check(sendto(m_sock, buf, len, 0, (sockaddr *)&m_modsServAddr, sizeof(m_modsServAddr))>=0,
			_T("Send Error: Send to MODS Server Error"));
	}
}


VOID CModsClient::ProcBuf(char * buf, int len) {
	memcpy(pStoreData, buf, len);
	nLenData = len;
}

VOID CModsClient::GetData(char * buf, int * len)
{
	CSingleLock csl(&CModsClient::m_recvLock);
	csl.Lock();	//ENTER CRITICAL REGION

	memcpy(buf+(*len), pStoreData, nLenData);
	*len += nLenData;
			
	csl.Unlock();
	return;
}

u_long CModsClient::GetLocalAddr()
{
	u_long selfAddr;
	char hostname[50];
	LPHOSTENT hs;
	gethostname(hostname, 50);
	hs = gethostbyname(hostname);
	memcpy((char *)&selfAddr, hs->h_addr_list[0], hs->h_length);
	return selfAddr;
}