/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"
#include "ToolBWServer.h"

#define THREAD_END_MAXWAIT			5
#define BANDWITH_SERVER_PORT		5602
#define MAX_STRING_SIZE				255

CToolBWServer::CToolBWServer(void)
{
	m_bIsStarted = false;
	status_thread = CLOSED;
	status_socket = DISCONNECTED;
}

CToolBWServer::~CToolBWServer(void)
{
	if (m_bIsStarted) Stop();
}

bool CToolBWServer::Start()
{
	if (m_bIsStarted) return true;

	StartWaitingThread();
	m_bIsStarted = true;
	return true;
}

bool CToolBWServer::Stop()
{
	if (!m_bIsStarted) return true;

	StopWaitingThread();
	m_bIsStarted = false;
	return true;
}

bool CToolBWServer::StartWaitingThread()
{
	StartNetwork();
	CreateSocket();
	m_threadWaiting = CreateThread(NULL, 0, EventWaitingThread, this, 0, NULL);
	status_thread	= RUNNING;
	return true;
}

bool CToolBWServer::StartNetwork()
{
	WSADATA wsaData;
	WORD wVersionRequested = MAKEWORD(2,2);
	int nRet;

	// Init WinSock
	nRet = WSAStartup(wVersionRequested, &wsaData);
	if (nRet != 0) {
		AfxMessageBox(_T("\nError initializing WinSock\n"));
		return FALSE;
	}

	// Check version
	if (wsaData.wVersion != wVersionRequested) {
		AfxMessageBox(_T("\nWinSock version not supported\n"));
		WSACleanup();
		return FALSE;
	}

	status_socket = INITIALIZED;
	return TRUE;
}

bool CToolBWServer::CreateSocket()
{
	if (status_socket < INITIALIZED) StartNetwork();
	m_sock = socket(AF_INET, SOCK_DGRAM, 0);
	if (m_sock == INVALID_SOCKET) {
		AfxMessageBox(_T("Socket Error: Can not create Socket!"));
		CloseNetwork();
		return FALSE;
	}
	status_socket = CREATED;

	//bind local address to socket
	memset(&m_localAddr,0,sizeof(m_localAddr));
	m_localAddr.sin_family = AF_INET;
	m_localAddr.sin_addr.s_addr = htonl(INADDR_ANY);
	m_localAddr.sin_port = htons(BANDWITH_SERVER_PORT);
	int nb = bind(m_sock,(struct sockaddr *)&m_localAddr,sizeof(m_localAddr));
	if (nb == SOCKET_ERROR) {
		AfxMessageBox(_T("Bind Error: Can not bind local address to Socket!"));
		CloseNetwork();
		return FALSE;
	}

	// set remote host information. Could be broadcast
	return TRUE;
}

DWORD WINAPI CToolBWServer::EventWaitingThread(LPVOID arg)
{
	CToolBWServer * thisClass = (CToolBWServer *)arg;
	thisClass->EventWaitingFun();
	return 0;
}

void CToolBWServer::EventWaitingFun()
{
	char buff[MAX_STRING_SIZE];

	struct sockaddr recvaddr;
	int rcvAddrLen = sizeof(recvaddr);
	int rcvLen;

	while(status_thread == RUNNING) {
		rcvLen = recvfrom(m_sock, buff, MAX_STRING_SIZE, 0, (sockaddr *)&recvaddr, &rcvAddrLen);
		if (status_thread != RUNNING) break;
		if (rcvLen ==SOCKET_ERROR || rcvLen <=0) break;
		
		sendto(m_sock, buff, rcvLen, 0, (sockaddr *)&recvaddr, sizeof(recvaddr));
	}

	status_thread = CLOSING;

	if(status_socket > DISCONNECTED)	CloseNetwork();

	// Self terminate the thread
	m_threadWaiting = NULL;
	status_thread = CLOSED;
}

void CToolBWServer::StopWaitingThread()
{
	CloseNetwork();

	int watchDog = 0;
	while (m_threadWaiting)
	{
		status_thread = CLOSING;
		SetThreadPriority(m_threadWaiting, THREAD_PRIORITY_HIGHEST);
		if((::WaitForSingleObject(m_threadWaiting, 1000) != WAIT_TIMEOUT) ||
			watchDog >= THREAD_END_MAXWAIT)
		{
			CloseHandle(m_threadWaiting);
			m_threadWaiting = NULL;
			status_thread = CLOSED;
		}
		watchDog++;
	}
}

void CToolBWServer::CloseNetwork()
{
	if (status_socket > INITIALIZED) CloseSocket();
	if (status_socket > DISCONNECTED) {
		WSACleanup();
		status_socket = DISCONNECTED;
	}
}

void CToolBWServer::CloseSocket()
{
	if (status_socket >= CREATED) {
		shutdown(m_sock, SD_BOTH);
		closesocket(m_sock);
		status_socket = INITIALIZED;
	}
}
