/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
#ifndef _TREE_INF_H
#define _TREE_INF_H

#define MAX_STRING_SIZE 255
#define MAX_BUFF_SIZE	65535

#include <string.h>

class CWord
{
public:
	/*the construction function of the class, when we need
	to compare tokens, we can compare the token with
	a "char", a "Word" or a "char *", it can be converted
	to a Word type automatically  */
	CWord(char *ch="") { strcpy_s(str, strlen(ch)+1, ch); }
	CWord(char ch) { str[0] = ch; str[1] = '\0'; }
	CWord(const CWord &w) {	strcpy_s(str, strlen(w.str)+1, w.str); }


	/*the construction function of operator '='*/
	CWord & operator = (const CWord &w) {
		if(this!=&w) strcpy_s(str, strlen(w.str)+1, w.str);
		return *this;
	}

	/*the construction function of comparing operators */
	friend bool operator == (const CWord &a, const CWord &b) { return strcmp(a.str, b.str) == 0; }
	friend bool operator != (const CWord &a, const CWord &b) { return strcmp(a.str, b.str) != 0; }
	friend bool operator <= (const CWord &a, const CWord &b) { return strcmp(a.str, b.str) <= 0; }
	friend bool operator >= (const CWord &a, const CWord &b) { return strcmp(a.str, b.str) >= 0; }
	friend bool operator < (const CWord &a, const CWord &b)  { return strcmp(a.str, b.str) < 0; }
	friend bool operator > (const CWord &a, const CWord &b)  { return strcmp(a.str, b.str) > 0; }
	
	char *get() { return str; }
private:
	/* the only member to store the token */
	char str[MAX_STRING_SIZE];
};

class CCMId {
public:
	CCMId(char *ch="", float v = 0.0) {
		strcpy_s(name, MAX_STRING_SIZE, ch);
		version = v;
	}
	CCMId(const CCMId &w) {
		strcpy_s(name, MAX_STRING_SIZE, w.name);
		version = w.version;
	}
	CCMId & operator = (const CCMId &w) {
		if (this != &w) {
			strcpy_s(name, MAX_STRING_SIZE, w.name);
			version = w.version;
		}
		return *this;
	}
	
	/*the construction function of comparing operators */
	friend bool operator == (const CCMId &a, const CCMId &b) {
		return (strcmp(a.name, b.name)==0 && a.version==b.version);
	}
	friend bool operator != (const CCMId &a, const CCMId &b) {
		return (strcmp(a.name, b.name)!=0 || a.version!=b.version);
	}
	friend bool operator <= (const CCMId &a, const CCMId &b) {
		return (strcmp(a.name, b.name)<0 || (strcmp(a.name, b.name)==0 && a.version<=b.version));
	}
	friend bool operator >= (const CCMId &a, const CCMId &b) {
		return (strcmp(a.name, b.name)>0 || (strcmp(a.name, b.name)==0 && a.version>=b.version));
	}
	friend bool operator < (const CCMId &a, const CCMId &b) {
		return (strcmp(a.name, b.name)<0 || (strcmp(a.name, b.name)==0 && a.version<b.version));
	}
	friend bool operator > (const CCMId &a, const CCMId &b) {
		return (strcmp(a.name, b.name)>0 || (strcmp(a.name, b.name)==0 && a.version>b.version));
	}

	char *	GetName() {return name;}
	float	GetVersion() {return version;}

	void	toString(char * str) {
		*(float *)str = version;
		strcpy_s(str+sizeof(version), MAX_STRING_SIZE-sizeof(version), name);
	}
	void	fromString(char * str) {
		version = *(float *)str;
		strcpy_s(name, MAX_STRING_SIZE, str+sizeof(version));
	}

private:
	char name[MAX_STRING_SIZE];
	float version;
	unsigned int cid;	// current do not use this info
};

typedef struct {
	CWord key;
	CWord value;
} COMPNTPARAM;

#endif //_TREE_INF_H