/**
 * Copyright (c) 2005 - Lehigh University. Bethlehem, PA, USA.
 * All rights reserved.
 * This source code is a part of MARCHES Project. 
 * 
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice, the following
 * two paragraphs, and the author attribution appear in all copies of this
 * software.
 *
 * IN NO EVENT SHALL LEHIGH UNIVERSITY BE LIABLE TO ANY PARTY FOR DIRECT, 
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF LEHIGH
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *	
 * LEHIGH UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, 
 * AND LEHIGH UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE, 
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 **/


////////////////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "modsCommon.h"
#include "modsdefn.h"

VOID MIBEntryInit(MIB_ENTRY* pent)
{
	pent->asnOid.idLength	= 0;
	pent->chType			= MODS_NULL;
	pent->pMibNext			= NULL;
	pent->pMibSon			= NULL;
}

VOID MIBEntryRelease(MIB_ENTRY* pent)
{
	if (pent->asnOid.idLength > 0)
		delete[] (pent->asnOid.ids);
	switch (pent->chType) {
		case MODS_INTEGER:
			delete ((ModsInteger *)pent->pStorageValue);
			break;
		case MODS_STRING:
			// how to process it ?
			break;
		case MODS_POSITION:
			delete ((ModsPosition *)pent->pStorageValue);
			break;
		case MODS_FIRE:
			delete ((ModsFire *)pent->pStorageValue);
			break;
		case MODS_FIREFIRGHTER:
			delete ((ModsSFighter *)pent->pStorageValue);
			break;
		case MODS_VIDEO:
			delete ((ModsVideo *)pent->pStorageValue);
			break;
		case MODS_CHATBOX:
			delete ((ModsCBox *)pent->pStorageValue);
			break;
	}
	delete pent;
}

VOID MIBTableRelease(MIB_ENTRY* pent)
{
	MIB_ENTRY *pTemp;
	if (pent == NULL) return;
	if (pent->pMibSon != NULL)
		MIBTableRelease(pent->pMibSon);
	if (pent->pMibNext != NULL) {
		pTemp = pent;
		pent  = pTemp->pMibNext;
		MIBEntryRelease(pTemp);
		MIBTableRelease(pent);
	}
	else MIBEntryRelease(pent);
}

// find the entry with 'id' in the current mib table
MIB_ENTRY* MODSFindEntry(MIB_ENTRY* pTable, MODSIdentifier id)
{
	MIB_ENTRY* pNext;

	pNext = pTable;
	while (pNext != NULL) {
		int nComp = MIBIDCompare(pNext->asnOid, id);
		if (nComp == 0) // current entry is the result
			return pNext;
		else if (nComp > 0) //the result is in the next level directory
			return MODSFindEntry(pNext->pMibSon, id);
		else //nComp < 0, the result is not in current directory
			pNext = pNext->pMibNext;
	}
	return NULL;
}

MIB_ENTRY* MODSFindParent(MIB_ENTRY* pTable, MODSIdentifier id)
{
	MIB_ENTRY *pResult;
	MODSIdentifier parentId;
	parentId.idLength = id.idLength - 1;
	parentId.ids = new UINT[parentId.idLength];
	memcpy(parentId.ids, id.ids, sizeof(parentId.ids[0])*parentId.idLength);
	pResult = MODSFindEntry(pTable, parentId);
	delete[] parentId.ids;

	return pResult;
}

MIB_ENTRY* MODSFindPrev(MIB_ENTRY* pTable, MODSIdentifier id)
{
	if (pTable == NULL) return NULL;
	MIB_ENTRY *pParent, *pPrev, *pNext;
	pParent = MODSFindParent(pTable, id);
	if (pParent == NULL) return NULL;
	pPrev = pParent;
	pNext = pPrev->pMibSon;
	if (pNext == NULL) return NULL;
	while (MIBIDCompare(pNext->asnOid, id) != 0) {
		pPrev = pNext;
		pNext = pPrev->pMibNext;
		if (pNext == NULL) return NULL;
	}
	return pPrev;
}

int MIBIDCompare(MODSIdentifier tbId, MODSIdentifier fId)
{
	UINT i;
	if (tbId.idLength < fId.idLength) {
		for (i = 0; i<tbId.idLength; i++)
			if (tbId.ids[i] != fId.ids[i]) return -1;
		return 1;
	}
	else if (tbId.idLength == fId.idLength) {
		for (i = 0; i<tbId.idLength; i++)
			if (tbId.ids[i] != fId.ids[i]) return -1;
		return 0;
	}
	return -1;
}

VOID MODSAddEntry(MIB_ENTRY* parent, MIB_ENTRY* pent)
{
	MIB_ENTRY *prv, *cur = new MIB_ENTRY;

	*cur = (*pent);
	cur->asnOid.idLength = parent->asnOid.idLength + 1;
	cur->asnOid.ids = new UINT[cur->asnOid.idLength];
	memcpy(cur->asnOid.ids, parent->asnOid.ids, sizeof(cur->asnOid.ids[0])*parent->asnOid.idLength);

	switch (pent->chType) {
		case MODS_INTEGER:
			cur->pStorageValue = new ModsInteger;
			memcpy(cur->pStorageValue, pent->pStorageValue, sizeof(ModsInteger));
			break;
		case MODS_STRING:
			// string is reserved!
			break;
		case MODS_POSITION:
			cur->pStorageValue = new ModsPosition;
			memcpy(cur->pStorageValue, pent->pStorageValue, sizeof(ModsPosition));
			break;
		case MODS_FIRE:
			cur->pStorageValue = new ModsFire;
			memcpy(cur->pStorageValue, pent->pStorageValue, sizeof(ModsFire));
			break;
		case MODS_FIREFIRGHTER:
			cur->pStorageValue = new ModsSFighter;
			memcpy(cur->pStorageValue, pent->pStorageValue, sizeof(ModsSFighter));
			break;
		case MODS_VIDEO:
			cur->pStorageValue = new ModsVideo;
			memcpy(cur->pStorageValue, pent->pStorageValue, sizeof(ModsVideo));
			break;
		case MODS_CHATBOX:
			cur->pStorageValue = new ModsCBox;
			memcpy(cur->pStorageValue, pent->pStorageValue, sizeof(ModsCBox));
			break;
	}
	
	*((ModsInteger *)parent->pStorageValue) += 1;
	if (parent->pMibSon == NULL) {
		cur->asnOid.ids[cur->asnOid.idLength-1] = 1;
		parent->pMibSon = cur;
	}
	else {
		prv = parent->pMibSon;
		while (prv->pMibNext != NULL) prv = prv->pMibNext;
		cur->asnOid.ids[cur->asnOid.idLength-1] = prv->asnOid.ids[prv->asnOid.idLength-1] + 1;
		prv->pMibNext = cur;
	}
}

VOID MODSEditEntry(MIB_ENTRY* root, MIB_ENTRY* pent)
{
	MIB_ENTRY* pcur = MODSFindEntry(root, pent->asnOid);
	if (pcur == NULL) return;
	switch (pent->chType) {
		case MODS_INTEGER:
			memcpy(pcur->pStorageValue, pent->pStorageValue, sizeof(ModsInteger));
			break;
		case MODS_STRING:
			// string is reserved!
			break;
		case MODS_POSITION:
			memcpy(pcur->pStorageValue, pent->pStorageValue, sizeof(ModsPosition));
			break;
		case MODS_FIRE:
			memcpy(pcur->pStorageValue, pent->pStorageValue, sizeof(ModsFire));
			break;
		case MODS_FIREFIRGHTER:
			memcpy(pcur->pStorageValue, pent->pStorageValue, sizeof(ModsSFighter));
			break;
		case MODS_VIDEO:
			memcpy(pcur->pStorageValue, pent->pStorageValue, sizeof(ModsVideo));
			break;
		case MODS_CHATBOX:
			memcpy(pcur->pStorageValue, pent->pStorageValue, sizeof(ModsCBox));
			break;
	}
}

VOID MODSDelEntry(MIB_ENTRY* root, MIB_ENTRY* pent)
{
	MIB_ENTRY* pPrev = MODSFindPrev(root, pent->asnOid);
	if (pPrev == NULL) return;
	MIB_ENTRY* pTemp = MODSFindParent(root, pent->asnOid);
	*((ModsInteger *)pTemp->pStorageValue) -= 1;

	ASSERT(pPrev != root); // Assume root can not be deleted
	ASSERT(pPrev->asnOid.idLength <= pent->asnOid.idLength);

	if (pPrev->asnOid.idLength < pent->asnOid.idLength) {// pent is the first entry
		pTemp = pPrev->pMibSon;
		pPrev->pMibSon = pTemp->pMibNext;
	}
	else { //pPrev->asnOid.idLength == pent->asnOid.idLength //same level
		pTemp = pPrev->pMibNext;
		pPrev->pMibNext = pTemp->pMibNext;
	}
	MIBTableRelease(pTemp->pMibSon);
	MIBEntryRelease(pTemp);
}

VOID MODSSetBuffer(MIB_ENTRY* pent, char * buf, UINT * slen)
{
//	short	packtype;
//	uint	identifierlen;
//	uint[]	identifiers;
//	short	valuetype;
//	ModsType data;

	memcpy(buf+(*slen), &(pent->asnOid.idLength), sizeof(pent->asnOid.idLength));
	(*slen) += sizeof(pent->asnOid.idLength);
	memcpy(buf+(*slen), pent->asnOid.ids, sizeof(pent->asnOid.ids[0])*pent->asnOid.idLength);

	(*slen) += sizeof(pent->asnOid.ids[0])*pent->asnOid.idLength;
	memcpy(buf+(*slen), &(pent->chType), sizeof(pent->chType));
	(*slen) += sizeof(pent->chType);
	switch (pent->chType) {
		case MODS_INTEGER:
			memcpy(buf+(*slen), pent->pStorageValue, sizeof(ModsInteger));
			(*slen) += sizeof(ModsInteger);
			break;
		case MODS_STRING:
			// string is reserved!
			break;
		case MODS_POSITION:
			memcpy(buf+(*slen), pent->pStorageValue, sizeof(ModsPosition));
			(*slen) += sizeof(ModsPosition);
			break;
		case MODS_FIRE:
			memcpy(buf+(*slen), pent->pStorageValue, sizeof(ModsFire));
			(*slen) += sizeof(ModsFire);
			break;
		case MODS_FIREFIRGHTER:
			memcpy(buf+(*slen), pent->pStorageValue, sizeof(ModsFFighter));
			(*slen) += sizeof(ModsFFighter);
			break;
		case MODS_VIDEO:
			memcpy(buf+(*slen), pent->pStorageValue, sizeof(ModsVideo));
			(*slen) += sizeof(ModsVideo);
			break;
		case MODS_CHATBOX:
			memcpy(buf+(*slen), pent->pStorageValue, sizeof(ModsCBox));
			(*slen) += sizeof(ModsCBox);
			break;
	}
}

VOID MODSSetTrap(MIB_ENTRY* pent, char * buf, UINT * slen, UINT *pntrap)
{
	if (pent == NULL) return;
	if (pent->unProperty == MODS_PROPERTY_TRAP) {
		MODSSetBuffer(pent, buf, slen);
		*pntrap += 1;
	}
	MODSSetTrap(pent->pMibSon, buf, slen, pntrap);
	MODSSetTrap(pent->pMibNext, buf, slen, pntrap);
}

UINT * MODSSetDeviceOid() {
	UINT * Oid	= new UINT[MODS_DEVICE_OID_LENGTH];	//	= {0,1};
	Oid[0]	= 0;
	Oid[1]	= 1;
	return Oid;
}

UINT * MODSSetVideoOid()
{
	UINT * Oid	= new UINT[MODS_VIDEO_OID_LENGTH];	//	= {0,1,1};
	Oid[0]	= 0;
	Oid[1]	= 1;
	Oid[2]	= 1;
	return Oid;
}

UINT * MODSSetChatBoxOid()
{
	UINT * Oid	= new UINT[MODS_CHATBOX_OID_LENGTH];//	= {0,1,2};
	Oid[0]	= 0;
	Oid[1]	= 1;
	Oid[2]	= 2;
	return Oid;
}

UINT * MODSSetDisplayOid()
{
	UINT * Oid	= new UINT[MODS_DISPLAY_OID_LENGTH];//	= {0,2};
	Oid[0]	= 0;
	Oid[1]	= 2;
	return Oid;
}

UINT * MODSSetFireOid()
{
	UINT * Oid	= new UINT[MODS_FIRE_OID_LENGTH];	//	= {0,2,1};
	Oid[0]	= 0;
	Oid[1]	= 2;
	Oid[2]	= 1;
	return Oid;
}

UINT * MODSSetFFighterOid()
{
	UINT * Oid	= new UINT[MODS_FIGHTER_OID_LENGTH];//	= {0,2,2};
	Oid[0]	= 0;
	Oid[1]	= 2;
	Oid[2]	= 2;
	return Oid;
}

UINT * MODSSetMessageOid()
{
	UINT * Oid	= new UINT[MODS_MESSAGE_OID_LENGTH];//	= {0,3};
	Oid[0]	= 0;
	Oid[1]	= 3;
	return Oid;
}

UINT * MODSSetPathOid()
{
	UINT * Oid	= new UINT[MODS_PATH_OID_LENGTH];	//	= {0,3,1};
	Oid[0]	= 0;
	Oid[1]	= 3;
	Oid[2]	= 1;
	return Oid;
}

MODSIdentifier MODSSetFireEntry()
{
	MODSIdentifier id;
	id.idLength = MODS_FIRE_OID_LENGTH + 1;
	id.ids		= new UINT[id.idLength];	//	= {0,2,1,? (0)};
	id.ids[0]	= 0;
	id.ids[1]	= 2;
	id.ids[2]	= 1;
	id.ids[3]	= -1;
	return id;
}

BOOL IsMODSFireEntry(MODSIdentifier id)
{
	if (id.idLength == MODS_FIRE_OID_LENGTH + 1 &&
		id.ids[0]	== 0 &&
		id.ids[1]	== 2 &&
		id.ids[2]	== 1)
		return TRUE;
	return FALSE;
}

MODSIdentifier MODSSetFFighterEntry()
{
	MODSIdentifier id;
	id.idLength = MODS_FIGHTER_OID_LENGTH + 1;
	id.ids		= new UINT[id.idLength];	//	= {0,2, 2,? (0)};
	id.ids[0]	= 0;
	id.ids[1]	= 2;
	id.ids[2]	= 2;
	id.ids[3]	= -1;
	return id;
}

BOOL IsMODSFFighterEntry(MODSIdentifier id)
{
	if (id.idLength == MODS_FIGHTER_OID_LENGTH + 1 &&
		id.ids[0]	== 0 &&
		id.ids[1]	== 2 &&
		id.ids[2]	== 2)
		return TRUE;
	return FALSE;
}

MODSIdentifier MODSSetVideoEntry()
{
	MODSIdentifier id;
	id.idLength = MODS_VIDEO_OID_LENGTH + 1;
	id.ids		= new UINT[id.idLength];	//	= {0,1,1,? (0)};
	id.ids[0]	= 0;
	id.ids[1]	= 1;
	id.ids[2]	= 1;
	id.ids[3]	= -1;
	return id;
}

BOOL IsMODSVideoEntry(MODSIdentifier id)
{
	if (id.idLength == MODS_VIDEO_OID_LENGTH + 1 &&
		id.ids[0]	== 0 &&
		id.ids[1]	== 1 &&
		id.ids[2]	== 1)
		return TRUE;
	return FALSE;
}

MODSIdentifier MODSSetChatBoxEntry()
{
	MODSIdentifier id;
	id.idLength = MODS_CHATBOX_OID_LENGTH + 1;
	id.ids		= new UINT[id.idLength];	//	= {0,1, 2,? (0)};
	id.ids[0]	= 0;
	id.ids[1]	= 1;
	id.ids[2]	= 2;
	id.ids[3]	= -1;
	return id;
}

BOOL IsMODSChatBoxEntry(MODSIdentifier id)
{
	if (id.idLength == MODS_CHATBOX_OID_LENGTH + 1 &&
		id.ids[0]	== 0 &&
		id.ids[1]	== 1 &&
		id.ids[2]	== 2)
		return TRUE;
	return FALSE;
}

MODSIdentifier MODSSetPathEntry()
{
	MODSIdentifier id;
	id.idLength = MODS_PATH_OID_LENGTH + 1;
	id.ids		= new UINT[id.idLength];	//	= {0,3, 1,? (0)};
	id.ids[0]	= 0;
	id.ids[1]	= 3;
	id.ids[2]	= 1;
	id.ids[3]	= -1;
	return id;
}

BOOL IsMODSPathEntry(MODSIdentifier id)
{
	if (id.idLength == MODS_PATH_OID_LENGTH + 1 &&
		id.ids[0]	== 0 &&
		id.ids[1]	== 3 &&
		id.ids[2]	== 1)
		return TRUE;
	return FALSE;
}

BOOL MODSCheckIdExist(MIB_ENTRY* g_entParent, MIB_ENTRY* pent)
{
	// current only process fire fighter entry
	if (!IsMODSFFighterEntry(pent->asnOid)) return false;

	if (g_entParent == NULL) return false;
	MIB_ENTRY* pNext = g_entParent->pMibSon;
	while (pNext != NULL) {
		if (((ModsSFighter*)pNext->pStorageValue)->ffighter.id == ((ModsSFighter*)pent->pStorageValue)->ffighter.id) {
			memcpy(pent->asnOid.ids, pNext->asnOid.ids, sizeof(pent->asnOid.ids[0])*pent->asnOid.idLength);
			return true;
		}
		pNext = pNext->pMibNext;
	}
	return false;
}